#!/usr/bin/env python
# -*- coding: utf-8 -*-

from shimehari.helpers import getEnviron

u"""
===============================
    Shimehari.configuration
    ~~~~~~~~~~~~~~~~~~~~~~~

    アプリケーション内の様々な設定や環境情報を保持する
    Config クラスを提供します。

===============================
"""

ENVIROMENTS = ['development', 'production']


class ConfigManager(object):
    u"""Config インスタンスを管理します。"""

    configrations = {}

    @classmethod
    def hasConfig(cls):
        u"""Config インスタンスが登録されているかどうか

        :param bool: Config インスタンスが登録されているかどうかを返します。
        """
        if not cls.configrations or len(cls.configrations) < 1:
            return False
        else:
            return True

    @classmethod
    def getConfigs(cls):
        u"""Config インスタンスが格納されたディクショナリを返します。"""
        return cls.configrations

    @classmethod
    def getConfig(cls, environ=None):
        u"""Config インスタンスを取得します。

        :param environ: 取得したい環境。
                        指定されない場合は現在の環境に合わせた Config が返されます。
        """
        if environ is None:
            environ = getEnviron()
        if environ in cls.configrations:
            return cls.configrations[environ]
        return None

    @classmethod
    def addConfigs(cls, configs=[]):
        u"""複数の Config インスタンスをまとめて登録します。

        :param configs: Config インスタンスを格納したディクショナリ
        """
        [cls.addConfig(c) for c in configs]

    @classmethod
    def addConfig(cls, config):
        u"""Config インスタンスを登録します。

        :param config: 登録したいコンフィグインスタンス
        """
        if isinstance(config, Config):
            cls.configrations.setdefault(config.environment, config)
        else:
            raise TypeError('config じゃない')

    @classmethod
    def removeConfig(cls, environment):
        u"""Config インスタンスを削除します。

        :param enviroment: 削除したい環境
        """
        if cls.configrations[environment]:
            cls.configrations.pop(environment)


u"""
===============================
    ::pkg:: Shimehari.coniguration

    Config
    ~~~~~~
    アプリケーションな設定や環境情報を保持する
    コンフィグクラス

===============================
"""
from datetime import timedelta


class Config(dict):
    u"""Config クラスはアプリケーションの設定を保存します。
    初期値として、以下の設定を保持しています。

    :param DEBUG: アプリケーションをデバッグモードとして起動するかどうか。
                  この項目が True に指定された場合、アプリケーションは常にデバッグモードで起動されます。
                  一時的にデバッグモードで起動したい場合は以下のコマンドを用いたほうが便利です。

                  ::

                    $ shimehari drink -d

    :param TEST:                        アプリケーションをテストモードとして起動するかどうか。
    :param APP_ROOT:                    アプリケーションのルートディレクトリを指定します。
    :param APP_DIRECTORY:               アプリケーションのファイルが置かれたディレクトリを指定します。
    :param MAIN_SCRIPT:                 アプリケーションで一番最初に実行されるスクリプトを指定します。
    :param APP_INSTANCE_NAME:           アプリケーションがインスタンス化される際の名前を指定します。
    :param CONTROLLER_DIRECTORY:        コントローラーが格納されたディレクトリを指定します。
    :param VIEW_DIRECTORY:              ビューファイルが格納されたディレクトリを指定します。
    :param MODEL_DIRECTORY:             モデルファイルが格納されたディレクトリを指定します。
    :param ASSETS_DIRECTORY:            アセットファイルが格納されたディレクトリを指定します。
    :param PREFERRED_URL_SCHEME:        URL を発行する際のプロトコルを指定します。
    :param AUTO_SETUP:                  自動的にセットアップを開始するかどうか指定します。
                                        この項目についての詳細は app.setup を参照してください。
    :param CONTROLLER_AUTO_NAMESPACE:   コントローラーディレクトリ内で、サブディレクトリが作成されていた場合、
                                        自動的にサブディレクトリ名を URL に反映させるかどうかを決定します。
    :param TEMPLATE_ENGINE:             デフォルトで使用するテンプレートエンジンを決定します。
    :param USE_X_SENDFILE:              静的ファイルを X-Sendfile を使用して返すかどうか指定します。
    :param SECRET_KEY:                  アプリケーション固有のシークレットキーです。アプリケーションごとに変更してください。
    :param SERVER_NAME:                 ホスト名を指定します。
    :param TRAP_HTTP_EXCEPTIONS:        hoge
    :param TRAP_BAD_REQUEST_ERRORS:     huga
    :param PRESERVE_CONTEXT_ON_EXCEPTION:   aaa
    :param SEND_FILE_MAX_AGE_DEFAULT:       静的ファイルのタイムアウトを指定します。
    :param PERMANENT_SESSION_LIFETIME:      セッションの有効期限を指定します。
    :param CACHE_STORE:                     標準で使用するキャッシュストアを指定します。
    :param CACHE_DEFAULT_TIMEOUT:           キャッシュとして保持するデフォルトの時間を指定します。
    :param CACHE_THRESHOLD:                 キャッシュの保持数を指定します。
    :param CACHE_KEY_PREFIX:                キャッシュとして格納するキーに対して接頭詞を指定します。
    :param CACHE_DIR:                       FileSystemCacheStore を使用する際、キャッシュの保存先を指定します。
    :param CACHE_OPTIONS:                   キャッシュストアインスタンス化時に渡すオプションを指定します。
    :param CACHE_ARGS:                      キャッシュストアインスタンス化時に渡す引数を指定します。
    :param LOG_FILE_OUTPUT:                 ログを外部ファイルとして出力するかどうか指定します。
    :param LOG_FILE_ROTATE:                 ログを外部ファイルとして出力する際、ログローテートするかどうか
    :param LOG_ROTATE_MAX_BITE:             ログローテートする場合、最大ファイルサイズを指定します。
    :param LOG_ROTATE_COUNT:                ログローテートする場合、過去の差分として何ファイルまで残すのか指定します。
    :param LOG_FILE_DIRECTORY:              ログファイルの出力先を指定します。
    :param LOG_DEBUG_FORMAT:                ログをコンソール上に出力する際のフォーマットを指定します。
    :param LOG_OUTPUT_FORMAT:               ログをファイルに出力する際のフォーマットを指定します。
    """
    defaults = {
        'DEBUG': False,
        'TEST': False,
        'APP_ROOT': None,
        'APP_DIRECTORY': 'app',
        'MAIN_SCRIPT': 'main',
        'APP_INSTANCE_NAME': 'app',
        'CONTROLLER_DIRECTORY': 'controllers',
        'VIEW_DIRECTORY': 'views',
        'MODEL_DIRECTORY': 'models',
        'ASSETS_DIRECTORY': 'assets',
        'PREFERRED_URL_SCHEME': 'http',
        'AUTO_SETUP': True,
        'CONTROLLER_AUTO_NAMESPACE': True,
        'TEMPLATE_ENGINE': 'jinja2',
        'USE_X_SENDFILE': False,
        'SECRET_KEY': '_secret_shimehari',
        'SERVER_NAME': None,
        'TRAP_HTTP_EXCEPTIONS': False,
        'TRAP_BAD_REQUEST_ERRORS': False,
        'PRESERVE_CONTEXT_ON_EXCEPTION': None,
        'SEND_FILE_MAX_AGE_DEFAULT': 12 * 60 * 60,
        'PERMANENT_SESSION_LIFETIME': timedelta(days=31),
        #キャッシュ
        'CACHE_STORE': None,
        'CACHE_DEFAULT_TIMEOUT': 300,
        'CACHE_THRESHOLD': 500,
        'CACHE_KEY_PREFIX': None,
        'CACHE_DIR': None,
        'CACHE_OPTIONS': None,
        'CACHE_ARGS': [],
        #ログ周り
        'LOG_FILE_OUTPUT': False,
        'LOG_FILE_ROTATE': False,
        'LOG_ROTATE_MAX_BITE': (5 * 1024 * 1024),
        'LOG_ROTATE_COUNT': 5,
        'LOG_FILE_DIRECTORY': 'app/log',
        'LOG_DEBUG_FORMAT': ('-' * 80 + '\n' +
        '%(levelname)s in %(module)s [%(pathname)s:%(lineno)d]:\n' +
        '%(message)s\n' +
        '-' * 80),
        'LOG_OUTPUT_FORMAT': (
        '%(asctime)s %(levelname)s in %(module)s [%(pathname)s:%(lineno)d]:\n' +
        '%(message)s\n' +
        '-' * 80)
    }

    def __init__(self, environment='development', defaults={}):
        d = self.defaults.copy()
        d.update(defaults)
        dict.__init__(self, d)
        self.environment = environment

    def dump(self):
        u"""現在の設定状況を出力します。"""
        _conf = ''
        for k, v in self.defaults.items():
            _conf += k + ' => ' + str(v) + '\n'
        return _conf



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.helpers
    ~~~~~~~~~~~~~~~~~
    便利メソッド
===============================
"""

import os
import sys
import re
import urlparse
import pkgutil
import mimetypes
from time import time
from zlib import adler32
from threading import RLock


u"""json が使用可能かどうか"""
jsonAvailable = True
json = None
try:
    import json
except ImportError:
    try:
        import simplejson as json
    except ImportError:
        try:
            from django.utils import simplejson as json
        except ImportError:
            jsonAvailable = False

from werkzeug.datastructures import Headers
from werkzeug.wsgi import wrap_file
from werkzeug.urls import url_quote
from werkzeug.routing import BuildError
from werkzeug.exceptions import NotFound
from shimehari.shared import currentApp, request, _appContextStack, _requestContextStack
from shimehari.core.helpers import importFromString


def _assertHaveJson():
    if not jsonAvailable:
        raise RuntimeError(u'json ないじゃん')

u"""redis が使用可能かどうか"""
redisAvailable = True
redis = None
try:
    import redis
except ImportError:
    redisAvailable = False


def _assertHaveRedis():
    if not redisAvailable:
        raise RuntimeError('no redis module found')


_osAltSep = list(sep for sep in [os.path.sep, os.path.altsep] if sep not in (None, '/'))

_missing = object()

_toJsonFilter = json.dumps


def getHandlerAction(resource, action):
    u"""与えられたコントローラーとアクション名からマッチするハンドラを返します。
    :param resource:    ハンドラを抱えているコントローラー
    :param action:      ハンドラを取得したいアクション
    """
    try:
        handler = getattr(resource, action)
        if callable(handler):
            return handler
    except AttributeError:
        pass


def getHostName(url):
    u"""URL からホスト名を取得します。

    :param url: ホスト名を抜き出したい URL

    """
    return urlparse.urlparse(url).netloc


import sgmllib


class Stripper(sgmllib.SGMLParser):
    u"""与えられたストリングから HTML タグを除去するクラス。"""
    def __init__(self):
        sgmllib.SGMLParser.___init__(self)

    def strip(self, htmlTag):
        self.theString = ""
        self.feed(htmlTag)
        self.close()
        return self.theString

    def handleData(self, data):
        self.theString += data


def urlFor(endpoint, **values):
    appctx = _appContextStack.top
    reqctx = _requestContextStack.top

    if appctx is None:
        raise RuntimeError('hoge')

    if reqctx is not None:
        urlAdapter = reqctx.urlAdapter
        # if urlAdapter is None:
            # raise RuntimeError('ooooo')
        if endpoint[:1] == ('.'):
            endpoint = endpoint[1:]
        external = values.pop('_external', False)
    else:
        urlAdapter = appctx.adapter
        if urlAdapter is None:
            raise RuntimeError('url adapter not found')
        external = values.pop('_external', True)

    anchor = values.pop('_anchor', None)
    method = values.pop('_method', None)
    appctx.app.injectURLDefaults(endpoint, values)

    try:
        rv = urlAdapter.build(endpoint, values, method=method, force_external=external)
    except BuildError, error:
        values['_external'] = external
        values['_anchor'] = anchor
        values['_method'] = method
        return appctx.app.handleBuildError(error, endpoint, **values)

    rv = urlAdapter.build(endpoint, values, method=method, force_external=external)

    if anchor is not None:
        rv += '#' + url_quote(anchor)
    return rv


def findPackage(importName):
    rootModName = importName.split('.')[0]
    loader = pkgutil.get_loader(rootModName)
    if loader is None or importName == '__main__':
        pkgPath = os.getcwd()
    else:
        if hasattr(loader, 'get_filename'):
            filename = loader.get_filename(rootModName)
        elif hasattr(loader, 'active'):
            filename = loader.active
        else:
            __import__(importName)
            filename = sys.modules[importName].__file__
        pkgPath = os.path.abspath(os.path.dirname(filename))

        if loader.is_package(rootModName):
            pkgPath = os.path.dirname(pkgPath)

    siteParent, siteFolder = os.path.split(pkgPath)
    pyPrefix = os.path.abspath(sys.prefix)
    if pkgPath.startswith(pyPrefix):
        return pyPrefix, pkgPath
    elif siteFolder.lower() == 'site-packages':
        parent, folder = os.path.split(siteParent)
        if folder.lower == 'lib':
            baseDir = parent
        elif os.path.basename(parent).lower() == 'lib':
            baseDir = os.path.dirname(parent)
        else:
            baseDir = siteParent
        return baseDir, pkgPath
    return None, pkgPath


u"""-----------------------------
    ::pkg:: Shimehari.helpers
    getModulesFromPyFile
    ~~~~~~~~~~~~~~~~~~~~

    Python ファイルからモジュールをインポートします
    [args]
    :targetPath
        インポートしたいモジュール名
    :rootPath

    [return]
    :modules
        インポートされたモジュール

----------------------------------"""
import glob


def getModulesFromPyFile(targetPath, rootPath):
    files = glob.glob(targetPath + '/*.py')
    modules = []
    for fn in files:
        if not '__init__' in fn:
            fn = fn.replace(rootPath, '')[1:].replace('.py', '').replace('/', '.')
            modules.append(fn)
    return modules


u"""-----------------------------
    ::pkg:: Shimehari.helpers
    getRootPath
    ~~~~~~~~~~~

    root パスを取得します
    [args]
    :importName
        インポート時の名前

    [return]
        :string ルートパス

----------------------------------"""


def getRootPath(importName):
    loader = pkgutil.get_loader(importName)
    if loader is None or importName == '__main__':
        return os.getcwd()

    if hasattr(loader, 'get_filename'):
        filePath = loader.get_filename(importName)
    else:
        __import__(importName)
        filePath = sys.modules[importName].__file__
    return os.path.dirname(os.path.abspath(filePath))


u"""-----------------------------
    ::pkg:: Shimehari.helpers
    getEnviron
    ~~~~~~~~~~

    現在の動作環境を取得します。
    [return]
    :url
        ホスト名を抜き出したい URL

----------------------------------"""


def getEnviron():
    if 'SHIMEHARI_WORK_ENV' in os.environ:
        return os.environ['SHIMEHARI_WORK_ENV']
    return 'development'


def sendFromDirectory(directory, filename, **options):
    filename = safeJoin(directory, filename)
    if not os.path.isfile(filename):
        print filename
        raise NotFound()
    options.setdefault('conditional', True)
    return sendFile(filename, **options)


def safeJoin(directory, filename):
    filename = os.path.normpath(filename)
    for sep in _osAltSep:
        if sep in filename:
            raise NotFound()
    if os.path.isabs(filename) or filename.startswith('../'):
        raise NotFound()
    return os.path.join(directory, filename)


def sendFile(filenameOrFp, mimetype=None, asAttachment=False,
             attachmentFilename=None, addEtags=True,
             cacheTimeout=None, conditional=False):
    u"""静的ファイルをレスポンスとして返します。

    :param filenameOrFp:    送りたいファイル名
    :param mimetype:        mimetype

    """
    mtime = None
    if isinstance(filenameOrFp, basestring):
        filename = filenameOrFp
        file = None
    else:
        from warnings import warn
        file = filenameOrFp
        filename = getattr(file, 'name', None)

        if not attachmentFilename and not mimetype \
           and isinstance(filename, basestring):
            warn(DeprecationWarning('THAAAAAAAA'), stacklevel=2)
        if addEtags:
            warn(DeprecationWarning('AAAAAAA?'), stacklevel=2)

    if filename is not None:
        if not os.path.isabs(filename):
            os.path.join(currentApp.rootPath, filename)
    if mimetype is None and (filename or attachmentFilename):
        mimetype = mimetypes.guess_type(filename or attachmentFilename)[0]
    if mimetype is None:
        mimetype = 'application/octet-stream'

    headers = Headers()
    if asAttachment:
        if attachmentFilename is None:
            if filename is None:
                raise TypeError('file name is invalid')
            attachmentFilename = os.path.basename(filename)
        headers.add('Content-Disposition', 'attachment', filename=attachmentFilename)

    if currentApp.useXSendFile and filename:
        if file is not None:
            file.close()
        headers['X-Sendfile'] = filename
        #for nginx
        #headers['X-Accel-Redirect'] = filename
        data = None
    else:
        if file is None:
            file = open(filename, 'rb')
            mtime = os.path.getmtime(filename)
        data = wrap_file(request.environ, file)

    rv = currentApp.responseClass(data, mimetype=mimetype, headers=headers, direct_passthrough=True)

    if mtime is not None:
        rv.last_modified = int(mtime)

    rv.cache_control.public = True
    if cacheTimeout is None:
        cacheTimeout = currentApp.getSendFileMaxAge(filename)
    if cacheTimeout:
        rv.cache_control.max_age = cacheTimeout
        rv.expires = int(time() + cacheTimeout)

    if addEtags and filename is not None:
        rv.set_etag('Shimehari-%s-%s-%s' %
            (os.path.getmtime(filename),
            os.path.getsize(filename),
            adler32(
                filename.encode('utf8') if isinstance(filename, unicode) else filename
            ) & 0xffffffff
            )
        )

    if conditional:
        rv = rv.make_conditional(request)
        if rv.status_code == 304:
            rv.headers.pop('x-sendfile', None)
    return rv


def jsonify(*args, **kwargs):
    if __debug__:
        _assertHaveJson()
    """
    if 'padded' in kwargs:
        if isinstance(kwargs['padded'], str):
            callback = request.args.get(kwargs['padded']) or 'jsonp'
        else:
            callback = request.args.get('callback') or request.args.get('jsonp') or 'jsonp'
        del kwargs['padded']
        jsonStr = json.dumps(dict(*args,**kwargs), indent=None)
        content = str(callback) + '(' + jsonStr + ')'
        return currentApp.responseClass(content, mimetype='application/javascript')
    """
    resp = json.dumps(dict(*args, **kwargs), indent=None if request.is_xhr else 2)
    return currentApp.responseClass(resp, mimetype='application/json')


def fillSpace(text, length):
    if len(text) < length:
        num = length - len(text)
        for i in range(num):
            text += ' '
    return text


class lockedCachedProperty(object):
    def __init__(self, func, name=None, doc=None):
        self.__name__ = name or func.__name__
        self.__module__ = func.__module__
        self.__doc__ = doc or func.__doc__
        self.func = func
        self.lock = RLock()

    def __get__(self, obj, type=None):
        if obj is None:
            return self
        with self.lock:
            value = obj.__dict__.get(self.__name__, _missing)
            if value is _missing:
                value = self.func(obj)
                obj.__dict__[self.__name__] = value
            return value


def getTemplater(app, templaterName, *args, **options):
    import shimehari
    templateDir = os.path.join(shimehari.__path__[0], 'template', templaterName)
    if not os.path.isdir(templateDir):
        raise ValueError('template engine not found :: %s\n' % templateDir)
    try:
        module = importFromString('shimehari.template.' + templaterName)
        templater = module.templater
    except Exception, e:
        raise Exception(e)
    return templater(app, *args, **options)


u"""--------------------------------------
    Shimehari.helpers._Kouzi
    ~~~~~~~~~~~~~~~~~~~~~~~~
    Shimehari のベーシッククラス
--------------------------------------"""


class _Kouzi(object):
    def __init__(self, importName, appFolder='app',
                  controllerFolder='controllers', viewFolder=None):
        self.importName = importName
        self.currentEnv = getEnviron()

        self.rootPath = getRootPath(self.importName)

        from shimehari.configuration import ConfigManager
        config = ConfigManager.getConfig(self.currentEnv)

        if config and self.rootPath == os.getcwd() + '/' + config['APP_DIRECTORY']:
            self.rootPath = os.getcwd()

        self.appFolder = config['APP_DIRECTORY'] if config and config['APP_DIRECTORY'] else appFolder
        self.controllerFolder = config['CONTROLLER_DIRECTORY'] if config and \
                                config['CONTROLLER_DIRECTORY'] else controllerFolder
        self.viewFolder = config['VIEW_DIRECTORY'] if config and config['VIEW_DIRECTORY'] else viewFolder

        self._staticFolder = config['ASSETS_DIRECTORY'] if config and config['ASSETS_DIRECTORY'] else None
        self._staticURL = None

    def _getStaticFolder(self):
        if self._staticFolder is not None:
            return os.path.join(self.rootPath, self.appFolder, self._staticFolder)

    def _setStaticFolder(self, value):
        self._staticFolder = value
    staticFolder = property(_getStaticFolder, _setStaticFolder)
    del _getStaticFolder, _setStaticFolder

    def _getStaticURL(self):
        if self._staticURL is None:
            if self.staticFolder is None:
                return None
            return '/' + os.path.basename(self.staticFolder)

    def _setStaticURL(self, value):
        self._staticURL = value
    staticURL = property(_getStaticURL, _setStaticURL)
    del _getStaticURL, _setStaticURL

    @property
    def hasStaticFolder(self):
        return self.staticFolder is not None

    u"""-----------------------------
        ::pkg:: Shimehari.helpers._Kouzi
        sendStaticFile
        ~~~~~~~~~~~~~~
        静的ファイルをレスポンスとして返します。

        [args]
        :filename 要求するファイル名

        [return]
        :response レスポンス

    ----------------------------------"""
    def sendStaticFile(self, filename):
        if not self.hasStaticFolder:
            raise RuntimeError('static file folder has none.')
        cacheTimeout = self.getSendFileMaxAge(filename)
        return sendFromDirectory(self.staticFolder, filename, cacheTimeout=cacheTimeout)

    u"""-----------------------------
        ::pkg:: Shimehari.helpers._Kouzi
        getSendFileMaxAge
        ~~~~~~~~~~~~~~~~~
        静的ファイルをレスポンスとして返します。

        [args]
        :filename 要求するファイル名

        [return]
        :response レスポンス

    ----------------------------------"""
    def getSendFileMaxAge(self, filename):
        return currentApp.config['SEND_FILE_MAX_AGE_DEFAULT']

    def openFile(self, filename, mode='rb'):
        if mode not in ('r', 'rb'):
            raise ValueError('can not read file')
        return open(os.path.join(self.rootPath, filename), mode)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.core.manage.commands.help
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    help コマンド

    各コマンドモジュールは共通インターフェースとして
    Command クラスを持ちます。

===============================
"""

import sys
from pkgutil import walk_packages

from shimehari.core.manage import AbstractCommand
from shimehari.core.exceptions import CommandError
from shimehari.core.manage import commands
from shimehari.core.manage.AbstractCommand import command_dict


u"""
===============================
    ::pkg:: Shimehari.core.manage.commands.help
    Command
    ~~~~~~~
    コマンドの実装

===============================
"""


class Command(AbstractCommand):
    name = 'help'
    summary = "Show help on command (e.g. 'shimehari help drink')"
    usage = "Usage: %prog COMMAND [OPTIONS]"

    def __init__(self):
        super(Command, self).__init__()

    def handle(self, *args, **options):
        load_all_commands()

        if args:
            ## FIXME: handle errors better here
            command = args[0]
            if command not in command_dict:
                raise CommandError('No command with the name: %s' % command)
            command = command_dict[command]
            sys.stdout.write(command.summary + '\n\n')
            command.parser.print_help()
            return 0

        command = AbstractCommand()

        sys.stdout.write(command.summary + '\n\n')
        command.parser.print_help()

        sys.stdout.write('\nCommands available:\n')
        available_commands = list(set(command_dict.values()))
        available_commands.sort(key=lambda x: x.name)

        indent = 0
        for command in available_commands:
            if command.hidden:
                continue
            length = len(command.name)
            indent = length if indent < len(command.name) else indent

        for command in available_commands:
            if command.hidden:
                continue
            sys.stdout.write('  %-*s  %s\n' % (indent, command.name, command.summary))
        sys.stdout.write('\nFurther information:\n  http://shimehari.hageee.net\n')
        return 0

Command()


def load_command(name):
    full_name = 'shimehari.core.manage.commands.%s' % name
    if full_name in sys.modules:
        return
    try:
        __import__(full_name)
    except ImportError:
        pass


def load_all_commands():
    for name in command_names():
        load_command(name)


def command_names():
    names = set((pkg[1] for pkg in walk_packages(path=commands.__path__)))
    return list(names)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.crypt
    ~~~~~~~~~~~~~~~~~
    セキュアアアア
===============================
"""

import uuid
from hashlib import sha1
import datetime
import time

from werkzeug.exceptions import abort
from werkzeug.routing import NotFound

from shimehari import session, shared, request

_exemptions = []

"""
https://github.com/sjl/flask-csrf
"""


class CSRF(object):

    def __init__(self, app, csrfHandler=None):
        self.app = app
        self.csrfHandler = csrfHandler

    def checkCSRFExempt(self):
        try:
            dest = self.app.controllers.get(request.endpoint)
            shared._csrfExempt = dest in _exemptions
        except NotFound:
            shared._csrfExempt = False

    #token の有効期限チェック
    def checkCSRFExpire(self, token):
        csrfCreateAt = session.pop('_csrfTokenAdded', None)
        expire = self.app.config['CSRF_EXPIRE']
        now = datetime.datetime.now()
        currentTime = time.mktime(now.timetuple())
        term = currentTime - csrfCreateAt
        if term > expire:
            return False
        return True

    def csrfProtect(self):
        if not shared._csrfExempt:
            if request.method == 'POST':
                token = session.pop('_csrfToken', None)
                if not token or token != request.form.get('_csrfToken'):
                    if self.csrfHandler:
                        self.csrfHandler(*self.app.matchRequest())
                else:
                    if not self.checkCSRFExpire(token):
                        abort(400)


def generateCSRFToken():
    if '_csrfToken' not in session:
        session['_csrfToken'] = genereateToken()
        now = datetime.datetime.now() + datetime.timedelta()
        session['_csrfTokenAdded'] = time.mktime(now.timetuple())
    return session['_csrfToken']


import string
import random


def genereateToken():
    def randstr(n):
        return ''.join(random.choice(string.digits + string.letters) for i in xrange(n))
    token = sha1(session.sid + randstr(6)).hexdigest()
    token = uuid.uuid5(uuid.NAMESPACE_URL, token)
    return token



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.core.manage.commands.generate
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    アプリケーションモジュールを新たに作成する generate コマンド

    各コマンドモジュールは共通インターフェースとして
    Command クラスを持ちます。

===============================
"""
import os
import sys
import shutil

from optparse import make_option

import shimehari
from shimehari.core.exceptions import CommandError
from shimehari.core.helpers import importFromString
from shimehari.core.manage import CreatableCommand
from shimehari.helpers import getEnviron
from shimehari.configuration import ConfigManager, Config


u"""
===============================
    ::pkg:: Shimehari.core.manage.commands.generate
    Command
    ~~~~~~~

    コマンドの実装

===============================
"""


class Command(CreatableCommand):
    name = 'generate'
    summary = 'Generate Shimehari Modules'
    usage = "Usage: %prog MODULE_NAME [OPTIONS]"

    option_list = CreatableCommand.option_list + (
        make_option('--path', '-p', action='store', type='string', dest='path', help='generating target path'),
    )

    def __init__(self):
        super(Command, self).__init__()

    def handle(self, moduleType, name, *args, **options):
        if not moduleType == 'controller':
            raise CommandError('ない')

        path = options.get('path')
        if path is None:
            currentPath = os.getcwd()
            try:
                importFromString('config')
            except:
                sys.path.append(os.getcwd())
                try:
                    importFromString('config')
                except ImportError:
                    raise CommandError('config file is not found...')
            config = ConfigManager.getConfig(getEnviron())
            path = os.path.join(currentPath, config['APP_DIRECTORY'], config['CONTROLLER_DIRECTORY'])

        if not os.path.isdir(path):
            raise CommandError('Given path is invalid')

        ctrlTemplate = os.path.join(shimehari.__path__[0], 'core', 'conf', 'controller_template.org.py')

        name, filename = self.filenameValidation(path, name)
        newPath = os.path.join(path, filename)

        self.readAndCreateFileWithRename(ctrlTemplate, newPath, name)

    def filenameValidation(self, path, name):
        if name.endswith(('.pyc', '.pyo', '.py.class')):
            raise CommandError('invalid name....')
        if not name.endswith('.py'):
            filename = name + '.py'
        else:
            filename = name
            name = name.replace('.py', '')

        name = self.checkDirectory(path, name)

        import re
        if re.search(r"\W", name) or re.match(r"\d", name[0]):
            raise CommandError('file name is invalid')
        name = name[0].upper() + name[1:]
        return name, filename

    def checkDirectory(self, path, name):
        sep = None
        if '.' in name:
            sep = '.'
        elif '/' in name:
            sep = '/'
        else:
            pass

        if sep is not None:
            names = name.split(sep)
        name = names.pop()
        d = "/".join(names)
        td = os.path.join(path, d)
        if not os.path.isdir(td):
            try:
                os.makedirs(td)
            except (IOError, OSError), e:
                raise CommandError('invaild name... %s' % e)
        return name

    def readAndCreateFileWithRename(self, old, new, name):
        u"""指定されたディレクトリからテンプレートファイルを読み込み
        新たに生成したい指定ディレクトリへファイルを生成します。

        :param old: テンプレートファイルのパス
        :param new: 生成したいディレクトリへのパスとファイル名
        :param name: 変更したいクラス名
        """
        if os.path.exists(new):
            raise CommandError('Controller already exists.')

        with open(old, 'r') as template:
            content = template.read()
            if '%s' in content:
                content = content % name
                content = content[3:]
                content = content[:len(content) - 3]
        with open(new, 'w') as newFile:
            newFile.write(content)
        sys.stdout.write("Genarating New Controller: %s\n" % new)

        try:
            shutil.copymode(old, new)
            self.toWritable(new)
        except OSError:
            sys.stderr.write('can not setting permission')

Command()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.routing
    ~~~~~~~~~~~~~~~~~
    ルーティング周り
===============================
"""
import re
import sys
from werkzeug.routing import Map, Rule, RuleTemplate

from shimehari.controllers import AbstractController
from shimehari.core import (RESTFUL_ACTIONS, ALLOWED_HTTP_METHOD_NAMES, RESTFUL_METHODS_MAP,
                            importFromString)
from shimehari.helpers import getHandlerAction, fillSpace
from shimehari.configuration import ConfigManager, Config


u"""
===============================
    Shimehari.routing.AbstractRouter
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    ルーター
===============================
"""


class AbstractRouter(Map):
    u"""Shimehari アプリケーションで使用するルーターの抽象クラス
    """
    def __init__(self, rules=[], defaultSubdomain='', charset='utf-8',
                strict_slashes=True, redirectDefaults=True,
                converters=None, sortParameters=False, sortKey=None,
                encodingErrors='replace', hostMatching=False):

        Map.__init__(self, rules=rules, default_subdomain=defaultSubdomain, charset=charset,
                    strict_slashes=strict_slashes, redirect_defaults=redirectDefaults,
                    converters=converters, sort_parameters=sortParameters, sort_key=sortKey,
                    encoding_errors=encodingErrors, host_matching=hostMatching)

        self._rules = []

        for rule in rules:
            self.add(rule)

    def dump(self):
        u"""
        現在定義されているルーティングを出力します。
        """
        _map = ''
        for rule in self._rules:
            method = list(rule.methods.copy())
            if 'HEAD' in method:
                method.remove('HEAD')
            _map += fillSpace(','.join(method), 15)
            _map += fillSpace(unicode(rule).encode('utf-8') + '  ', 50)
            _map += '[action => '
            _map += rule.endpoint.__name__
            clsName = rule.endpoint.im_class.__name__
            _map += ', controller => ' + clsName + ']\n'
        return _map


class RESTfulRouter(AbstractRouter):
    u"""リソースを指定して自動的にルーティング生成


    :param resources:           ルーティングを登録したいリソース。
    :param defaultSubdomain:    prefix として指定するサブドメインを設定します。
    :param charset:             文字コードを設定します。
    :param strictSlashes:       ほげほげ
    :param redirectDefaults:    ほげほげ
    :param converters:          こんばーたーをしてい
    :param sortParameters:      ほげほげ
    :param sortKey:             ふがふが
    :param encordingErrors:     ほげほげ
    :param hostMatching:        ああああ

    """
    def __init__(self, resources=[], defaultSubdomain='', charset='utf-8',
                strict_slashes=True, redirectDefaults=True,
                converters=None, sortParameters=False, sortKey=None,
                encodingErrors='replace', hostMatching=False):

        AbstractRouter.__init__(self, rules=[], defaultSubdomain=defaultSubdomain, charset=charset,
                    strict_slashes=strict_slashes, redirectDefaults=redirectDefaults,
                    converters=converters, sortParameters=sortParameters, sortKey=sortKey,
                    encodingErrors=encodingErrors, hostMatching=hostMatching)

        self._rules = []

        for resource in resources:
            if isinstance(resource, (Resource, RESTfulRule)):
                for rule in resource._rules:
                    self.add(rule)
            elif isinstance(resource, list):
                for rule in resource:
                    self.add(rule)
            elif isinstance(resource, (Rule, Root)):
                self.add(resource)
            else:
                raise TypeError('resources rule is invalid.')


class Root(Rule):
    root = True

    def __init__(self, endpoint, methods=['get']):
        Rule.__init__(self, '/', endpoint=endpoint, methods=methods)


class RESTfulRule(object):

    parent = None

    baseName = None

    def __init__(self, name, index=None, show=None, edit=None, new=None, create=None, update=None, destroy=None, methods=[]):
        self.baseName = name
        self.getName = self.getNameFromRESTAction

        rules = []

        if index is not None:
            rules.append(Rule(self.getName(name, 'index'), endpoint=index, methods=['get']))
        if show is not None:
            rules.append(Rule(self.getName(name, 'show'), endpoint=show, methods=['get', 'post']))
        if edit is not None:
            rules.append(Rule(self.getName(name, 'edit'), endpoint=edit, methods=['get', 'post']))
        if new is not None:
            rules.append(Rule(self.getName(name, 'new'), endpoint=new, methods=['get']))
        if create is not None:
            rules.append(Rule(self.getName(name, 'create'), endpoint=create, methods=['post']))
        if update is not None:
            rules.append(Rule(self.getName(name, 'update'), endpoint=update, methods=['put']))
        if destroy is not None:
            rules.append(Rule(self.getName(name, 'destroy'), endpoint=destroy, methods=['delete']))
        self._rules = rules

    def refresh(self):
        for rule in self._rules:
            if self.parent:
                rule.rule = self.parent.baseName + '/' + rule.rule
            # rule.refresh()

    #uアクション名から URL Name を生成します
    def getNameFromRESTAction(self, name, action, root=False):
        if not name.startswith('/') and len(name) > 1:
            name = '/' + name
        if name.endswith('/') and len(name) > 1:
            name = name[:len(name) - 1]

        if root:
            name = ''

        _act = action.lower()
        if _act == 'index' or _act == 'create':
            if name == '' and root:
                return '/'
            return name
        elif _act == 'show' or _act == 'update' or _act == 'destroy':
            return name + '/<int:id>'
        elif _act == 'edit':
            return name + '/<int:id>/edit'
        elif _act == 'new':
            return name + '/new'
        else:
            raise ValueError('RESTfulRule')


class Resource(Map):
    u"""リソース

    :param controller:  リソースとして指定するコントローラー
    :param children:    aaa
    :param name:        namae
    :param only:

    """

    parent = None

    baseName = None

    orgName = None

    def __init__(self, controller=None, children=[], name=None, only=[], excepts=[], root=False,
                subdomain=None, buildOnly=False, strict_slashes=None, redirectTo=None,
                alias=False, host=None, defaults=None, namespace=None):

        Map.__init__(self, rules=[], default_subdomain='', charset='utf-8',
                 strict_slashes=True, redirect_defaults=True,
                 converters=None, sort_parameters=False, sort_key=None,
                 encoding_errors='replace', host_matching=False)

        if only != [] and excepts != []:
                raise ValueError('allow or deny!!!!')

        self._rules = []

        self.children = children

        self.only = only

        self.excepts = excepts

        self.root = root

        self.subdomain = self.default_subdomain = subdomain

        self.buildOnly = buildOnly

        self.strict_slashes = strict_slashes

        self.redirectTo = redirectTo

        self.alias = alias

        self.host = host

        self.defaults = defaults

        self.name = name

        self.getName = self.getNameFromRESTAction

        self.namespace = namespace
        if self.namespace:
            if self.namespace.startswith('/'):
                self.namespace = self.namespace[1:]
            if self.namespace.endswith('/'):
                self.namespace = self.namespace[:len(self.namespace) - 1]
        #ディレクトリが深かった時自動で
        elif controller is not None:
            config = ConfigManager.getConfig()
            if config['CONTROLLER_AUTO_NAMESPACE'] and type(controller) is not int:
                pkgs = controller.__module__.split('.')
                appFolder = config['APP_DIRECTORY']
                controllerFolder = config['CONTROLLER_DIRECTORY']
                if appFolder in pkgs:
                    pkgs.remove(appFolder)
                if controllerFolder in pkgs:
                    pkgs.remove(controllerFolder)

                self.orgName = pkgs[len(pkgs) - 1]
                pkgs = pkgs[:len(pkgs) - 1]
                self.namespace = "/".join(pkgs)

        if controller:
            controller = self._checkControllerType(controller)
            if not name:
                self.baseName = self.orgName = controller.baseName
            self.add(controller)

        #uuuumu
        if self.root:
            self.baseName = '/'

        if self.parent:
            self.baseName = self.parent.baseName + '/' + self.baseName

        if isinstance(self.children, dict) and len(self.children) > 0:
            for child in self.children:
                if not isinstance(child, Resource) and not isinstance(child, RESTfulRule) and not isinstance(child, Rule):
                    raise TypeError('children is invalid')
                if isinstance(child, Rule):
                    child.rule = self.baseName + '/' + child.rule
                else:
                    child.parent = self
                child.refresh()
                if isinstance(child, Resource):
                    self._rules = self._rules + child._rules

    def _checkControllerType(self, controller):
        from shimehari.controllers import AbstractController
        if isinstance(controller, str):
            controller = importFromString(controller)

        import inspect
        if inspect.isclass(controller):
            controller = controller(controller.__name__)
        if not isinstance(controller, AbstractController) and \
           not isinstance(controller, RESTfulRule):
            if isinstance(controller, dict):
                for rule in dict:
                    if not isinstance(rule, Rule):
                        raise ValueError('controller failed:: rule is invalid')
                        sys.exit()
            else:
                raise ValueError('controller is invalid... \n target instansce type is  %s' % type(controller))
        return controller

    #みなおし
    def _addRuleFromController(self, controller):
        actions = RESTFUL_ACTIONS.copy()
        if self.only is not None:
            actions = RESTFUL_ACTIONS.intersection(self.only)
        if self.excepts is not None:
            actions = RESTFUL_ACTIONS.difference(self.excepts)

        for action in actions:
            handler = getHandlerAction(controller, action)
            if handler is not None:
                name = self.getName(self.baseName, action, self.root)

                methods = RESTFUL_METHODS_MAP[action.lower()]
                rule = Rule(name, defaults=self.defaults, endpoint=handler,
                            subdomain=self.subdomain, methods=methods, build_only=self.buildOnly,
                            strict_slashes=self.strict_slashes, redirect_to=self.redirectTo, alias=self.alias, host=self.host)

                self._rules.append(rule)

    def _addRuleFromRules(self, rules):
        for rule in rules:
            if not isinstance(rule, Rule):
                raise TypeError('Rule type is invalid')
            self._rules.append(rule)

    def add(self, controller):
        u"""コントローラー、もしくはルールを追加します。


        :param controller:  ルールを追加したいコントローラー、もしくは
                            RESTfulRule、ルールを格納したディクショナリ

        """
        import inspect
        if inspect.isclass(controller):
            controller = controller(controller.__name__)
            self.baseName = controller.baseName
        if isinstance(controller, AbstractController):
            self._addRuleFromController(controller)
        elif isinstance(controller, RESTfulRule):
            self._rules = controller._rules
        elif isinstance(controller, dict):
            self._addRuleFromRules(controller)
        else:
            raise ValueError('out!')

    def refresh(self):
        for rule in self._rules:
            if self.parent:
                rule.rule = '/' + self.parent.baseName + rule.rule

    def addRule(self, name, handler, methods=[], *args, **kwargs):
        u"""リソースに対してルールを追加します。

        :param name:        名前
        :param handler:     処理
        :param methods:     使用する HTTP メソッド

        """
        rule = Rule(name, defaults=self.defaults, endpoint=handler,
                            subdomain=self.subdomain, methods=methods, build_only=self.buildOnly,
                            strict_slashes=self.strict_slashes, redirect_to=self.redirectTo, alias=self.alias, host=self.host)
        self._rules.append(rule)
        self.refresh()
        return self

    #上書きできるようにしよう
    def overRideRule(self, action, name, handler=None, methods=[], *args, **kwargs):
        print self._rules

    def getNameFromRESTAction(self, name, action, root=False):
        u"""アクション名から URL Name を生成します。

        :param name:    名前
        :param action:  アクション
        :param root:    ルートかどうか
        """

        if self.namespace:
            name = self.namespace + '/' + name

        if not name.startswith('/') and len(name) > 1:
            name = '/' + name
        if name.endswith('/') and len(name) > 1:
            name = name[:len(name) - 1]

        if root:
            name = ''

        _act = action.lower()
        if _act == 'index' or _act == 'create':
            if name == '' and root:
                return '/'
            return name
        elif _act == 'show' or _act == 'update' or _act == 'destroy':
            return name + '/<int:id>'
        elif _act == 'edit':
            return name + '/<int:id>/edit'
        elif _act == 'new':
            return name + '/new'
        else:
            raise ValueError('RESTfulRule')


u"""
werkzeug の中に似たのがあるっぽい…
"""


class Group(object):
    def __init__(self, name, children):
        for child in children:
            if isinstance(child, Resource):
                pass
                child.namespace = name
                child.refresh()
            elif isinstance(child, Rule):
                child.rule = name + '/' + child.name
                child.refresh()
            else:
                raise RuntimeError('settei dekimasen.')


#defaultRouterClass
Router = RESTfulRouter



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.core.manage.commands.create
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    アプリケーションを新たに作成する create コマンド

    各コマンドモジュールは共通インターフェースとして
    Command クラスを持ちます。

===============================
"""

import os
import sys
import errno
import shutil
from optparse import make_option

import shimehari
from shimehari.core.manage import CreatableCommand
from shimehari.core.helpers import importFromString
from shimehari.core.exceptions import CommandError

debugFormat = ('-' * 80 + '\\n' + '%(levelname)s in %(module)s [%(pathname)s:%(lineno)d]:\\n' + '%(message)s\\n' + '-' * 80)

outputFormat = ('%(asctime)s %(levelname)s in %(module)s [%(pathname)s:%(lineno)d]:\\n' + '%(message)s\\n' + '-' * 80)


u"""
===============================
    ::pkg:: Shimehari.core.manage.commands.create
    Command
    ~~~~~~~
    コマンドの実装

===============================
"""


class Command(CreatableCommand):
    name = 'create'
    summary = 'Create Shimehari Application'
    usage = "Usage: %prog APPLICATION_NAME [OPTIONS]"
    option_list = CreatableCommand.option_list + (
        make_option('--path', '-p', action='store', type='string', dest='path', help='target create path'),
        make_option('--template', '-t', action='store', type='string', dest='template', help='using project tempalte')
    )

    def __init__(self):
        super(Command, self).__init__()

    def handle(self, appDir='app', *args, **options):
        try:
            importFromString(appDir)
        except ImportError:
            pass
        else:
            raise CommandError('%s mou aru' % appDir)

        path = options.get('path')
        if path is None:
            appRootDir = os.path.join(os.getcwd(), appDir)
            try:
                os.makedirs(appRootDir)
            except OSError, error:
                if error.errno == errno.EEXIST:
                    msg = '%s is already exists' % appRootDir
                else:
                    msg = error
                raise CommandError(msg)
        else:
            appRootDir = os.path.abspath(os.path.expanduser(path))
            if not os.path.exists(appRootDir):
                raise CommandError("auau")

        template = options.get('template')
        if template is None:
            appTemplateDir = os.path.join(shimehari.__path__[0], 'core', 'conf', 'app_template')
        else:
            appTemplateDir = template

        prefixLen = len(appTemplateDir) + 1

        for root, dirs, files in os.walk(appTemplateDir):
            pathRest = root[prefixLen:]
            relativeDir = pathRest.replace('app_name', 'app')
            if relativeDir:
                targetDir = os.path.join(appRootDir, relativeDir)
                if not os.path.exists(targetDir):
                    os.mkdir(targetDir)

            for dirname in dirs[:]:
                if dirname.startswith('.'):
                    dirs.remove(dirname)

            for filename in files[:]:
                if filename.endswith(('.pyo', '.pyc', '.py.class')):
                    continue
                oldPath = os.path.join(root, filename)
                newPath = os.path.join(appRootDir, relativeDir, filename.replace('app_name', 'app'))

                self.readAndCreateFile(oldPath, newPath)

        #ここどうすっかな
        self.createDirectory(appRootDir, 'views')
        self.createDirectory(appRootDir, 'assets')
        self.createDirectory(appRootDir, 'log')

        #generate config file
        confOrgPath = os.path.join(shimehari.__path__[0], 'core', 'conf', 'config.org.py')
        newConfPath = os.path.join(os.getcwd(), 'config.py')
        self.readAndCreateFileWithRename(confOrgPath, newConfPath,
                                            (appDir, appDir, debugFormat, outputFormat))

        sys.stdout.write("New App Create Complete. enjoy!\n")

    u"""-----------------------------
        ::pkg:: Shimehari.core.manage.commands.create.Command
        readAndCreateFile
        ~~~~~~~~~~~~~~~~~

        指定されたディレクトリからテンプレートファイルを読み込み
        新たに生成したい指定ディレクトリへファイルを生成します。
        [args]
            :old テンプレートファイルのパス
            :new 生成したいディレクトリへのパスとファイル名
    ------------------------------"""
    def readAndCreateFile(self, old, new):
        if os.path.exists(new):
            raise CommandError('already... %s' % new)

        with open(old, 'r') as template:
            content = template.read()
        with open(new, 'w') as newFile:
            newFile.write(content)
        sys.stdout.write(u"Creating: %s\n" % new)

        try:
            shutil.copymode(old, new)
            self.toWritable(new)
        except OSError:
            sys.stderr.write('permission error')

    def readAndCreateFileWithRename(self, old, new, name):
        if os.path.exists(new):
            raise CommandError('Controller already exists.')

        with open(old, 'r') as template:
            content = template.read()
            if '%s' in content:
                content = content % name
        with open(new, 'w') as newFile:
            newFile.write(content)
        sys.stdout.write("Creating: %s\n" % new)

        try:
            shutil.copymode(old, new)
            self.toWritable(new)
        except OSError:
            sys.stderr.write('can not setting permission')

    def createDirectory(self, rootDir, dirname):
        targetName = os.path.join(rootDir, dirname)
        if not os.path.exists(targetName):
            os.mkdir(targetName)
            sys.stdout.write("Creating: %s\n" % targetName)

Command()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.session
    ~~~~~~~~~~~~~~~~~
    セッション管理
===============================
"""

from werkzeug.contrib.securecookie import SecureCookie
from werkzeug.contrib.sessions import SessionStore as SessionStoreBase, Session
from shimehari.helpers import jsonAvailable
from shimehari.core.helpers import importPreferredMemcachedClient


"""シリアライズで使用するモジュールを決定します。"""
try:
    import msgpack as msg
except ImportError:
    try:
        import cpickle as msg
    except ImportError:
        import pickle as msg
    finally:
        if not jsonAvailable:
            raise RuntimeError('you do not have selializer')
        from shimehari.helpers import json as msg


u"""
===============================
    ::pkg:: Shimehari.sessions
    SessionMixin
    ~~~~~~~~~~~~

    セッションの機能をもたせます

===============================
"""


class SessionMixin(object):
    def _get_permanent(self):
        return self.get('_permanent', False)

    def _set_permanent(self, value):
        self['_permanent'] = bool(value)

    permanent = property(_get_permanent, _set_permanent)
    del _get_permanent, _set_permanent

    new = False
    modified = True


u"""
===============================
    ::pkg:: Shimehari.sessions
    SecureCookieSession
    ~~~~~~~~~~~~~~~~~~~

    SecureCookie を使ったセッション

===============================
"""


class SecureCookieSession(SecureCookie, SessionMixin):
    u"""SecureCookie を使ったセッションクラス

    :param initial:     セッションデータ
    :param sid:         セッションID
    :param new:         新規セッション
    """
    def __init__(self, initial=None, sid=None, new=False):
        def on_update(self):
            self.modified = True
        SecureCookie.__init__(self, initial, on_update)
        self.sid = sid
        self.new = new
        self.modified = False


u"""
===============================
    ::pkg:: Shimehari.sessions
    NullSession
    ~~~~~~~~~~~

    物言わぬセッション

===============================
"""


class NullSession(Session):
    def _fail(self, *args, **kwargs):
        raise RuntimeError('Null Session!!!')

    __setitem__ = __delitem__ = clear = pop = popitem = setdefault = update = _fail
    del _fail


u"""
===============================
    ::pkg:: Shimehari.sessions
    _SessionStore
    ~~~~~~~~~~~~~

    werkzeug のセッションストアに多少機能を追加します。

===============================
"""


class _SessionStore(SessionStoreBase):
    nullSessionClass = NullSession

    def save(self, session, response):
        return NotImplementedError()

    def isNullSession(self, session):
        return isinstance(session, self.nullSessionClass)

    def makeNullSession(self):
        return self.nullSessionClass()


from pickle import HIGHEST_PROTOCOL


class MemcachedSessionStore(_SessionStore):
    u"""セッションの保存先に Memcache を使ったセッションストア

    :param servers:         Memcache サーバー
    :param keyPrefix:       セッション格納時、キーに付ける接頭詞
    :param defaultTimeout:  セッションの保存期間
    :param session_class:   セッションクラス
    """
    def __init__(self, servers=None, keyPrefix=None, defaultTimeout=300, session_class=None):
        _SessionStore.__init__(self, session_class=session_class)

        if isinstance(servers, (list, tuple)):
            client = importPreferredMemcachedClient(servers)
        elif servers is not None:
            client = servers
        else:
            raise RuntimeError('no memcache module.')

        self._memcacheClient = client
        self._memcacheKeyPrefix = keyPrefix
        self._memcacheTimeout = defaultTimeout

    def save(self, session, response):
        key = self._getMemcacheKey(session.sid)
        data = msg.dumps(dict(session))
        self._memcacheClient.set(key, data)

    def delete(self, session):
        self._memcacheClient.delete(self._getMemcacheKey(session.sid))

    def get(self, sid):
        if not self.is_vaild_key(sid):
            return self.new()

        key = self._getMemcacheKey(sid)
        packed = self._memcacheClient.get(key)
        try:
            data = msg.loads(packed)
        except TypeError:
            data = {}
        return self.session_class(data, sid, False)

    def _getMemcacheKey(self, sid):
        if self._memcacheKeyPrefix:
            key = self._memcacheKeyPrefix + sid
        else:
            key = sid
        if isinstance(key, unicode):
            key = key.encode('utf-8')
        return key


from shimehari.shared import request


class SecureCookieSessionStore(_SessionStore):
    u"""SecureCookieSession を利用したセッションストア"""
    def __init__(self, key='session', expire=0):
        _SessionStore.__init__(self, session_class=SecureCookieSession)
        self.key = key
        #直せ
        self.path = '/'
        #直せ
        self.domain = None

    def save(self, session, response):
        if session.modified and not session:
            response.delete_cookie(self.key, path=self.path, domain=self.domain)
        else:
            session.save_session(response, self.key, path=self.path, expire=self.expire, httponly=self.httponly,
                secure=self.secure, domain=self.domain)

    def delete(self, session):
        session.pop(self.key, None)

    #hum...
    def get(self, sid):
        if not self.is_vaild_key(sid):
            return self.new()

        if self.key is not None:
            return self.session_class.load_cookie(request, self.key, secret_key=self.key)


class RedisSessionStore(_SessionStore):
    u"""Redis を使ったセッションストア

    Author: @soundkitchen Izukawa Takanobu

    :param
    """
    def __init__(self, host='localhost', port=6379, db=0, expire=0, session_class=None):
        from redis import Redis
        super(RedisSessionStore, self).__init__(session_class=session_class)
        self._conn = Redis(host=host, port=port, db=db)
        self._expire = int(expire)

    def save(self, session):
        packed = msg.dumps(dict(session))
        self._conn.set(session.sid, packed)

    def delete(self, session):
        self._conn.delete(session.sid)

    def get(self, sid):
        if not self.is_valid_key(sid):
            return self.new()

        packed = self._conn.get(sid)
        try:
            data = msg.loads(packed, encoding='utf-8')
            if self._expire:
                self._conn.expire(sid, self._expire)
        except TypeError, e:
            data = {}
        return self.session_class(data, sid, False)


u"""デフォルトのセッションストアを決定します。"""
try:
    from werkzeug.contrib.sessions import FileSystemSessionStore
    _currentStore = FileSystemSessionStore()
except (Exception, RuntimeError), e:
    try:
        u"""GAE 対策"""
        _currentStore = MemcachedSessionStore()
    except (Exception, RuntimeError), e:
        u"""それでもダメだったら最後の手段"""
        _currentStore = SecureCookieSessionStore()
finally:
    SessionStore = _currentStore.__class__



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.contexts
    ~~~~~~~~~~~~~~~~~~
    コンテキスト周り
===============================
"""

import sys
from werkzeug.exceptions import HTTPException
from .shared import _requestContextStack, _appContextStack


u"""-----------------------------
    ::pkg:: Shimehari.contexts
    _getNecessaryAppContext
    ~~~~~~~~~~~~~~~~~~~~~~~

    スタックからアプリケーションコンテキストを取得します。

    ------------------------------"""


def _getNecessaryAppContext(app):
    top = _appContextStack.top
    if top is None or top.app != app:
        context = app.appContext()
        context.push()
        return context


u"""-----------------------------
    ::pkg:: Shimehari.contexts
    hasAppContext
    ~~~~~~~~~~~~~

    スタックにアプリケーションコンテキストが
    格納されているかどうか bool 値で返します。
    [return]
    :bool スタックにアプリケーションコンテキストが
          格納されているかどうか

    ------------------------------"""


def hasAppContext():
    return _appContextStack.top is not None


u"""-----------------------------
    ::pkg:: Shimehari.contexts
    hasRequestContext
    ~~~~~~~~~~~~~~~~~

    スタックにリクエストコンテキストが
    格納されているかどうか bool 値で返します。
    [return]
    :bool スタックにリクエストコンテキストが
          格納さているかどうか

    ------------------------------"""


def hasRequestContext():
    return _requestContextStack.top is not None


u"""
===============================
    ::pkg:: Shimehari.contexts
    AppContext
    ~~~~~~~~~~

    アプリケーションの情報を抽象化してとっておく

===============================
"""


class AppContext(object):
    def __init__(self, app):
        self.app = app
        self.adapter = app.createAdapter(None)

    u"""-----------------------------
        ::pkg:: Shimehari.contexts.AppContext
        push
        ~~~~

        現在のアプリケーションコンテキストをスタックに追加します。

    ------------------------------"""
    def push(self):
        _appContextStack.push(self)

    u"""-----------------------------
        ::pkg:: Shimehari.contexts.AppContext
        pop
        ~~~

        アプリケーションコンテキストをスタックから取り出します。

    ------------------------------"""
    def pop(self, exc=None):
        if exc is None:
            exc = sys.exc_info()[1]
        rv = _appContextStack.pop()
        assert rv is self, 'app context auau'

    def __enter__(self):
        self.push()
        return self

    def __exit__(self, exc_type, exc_value, tb):
        self.pop()


u"""
===============================
    ::pkg:: Shimehari.contexts
    RequestContext
    ~~~~~~~~~~~~~~

    リクエストオブジェクトを抽象化して保存します

===============================
"""


class RequestContext(object):
    def __init__(self, app, environ):
        self.app = app
        self.request = app.requestClass(environ)
        self.urlAdapter = app.createAdapter(self.request)
        self.flash = None
        self.session = None
        self.shared = app.sharedRequestClass()
        self.preserved = False
        self._pushedApplicationContext = None

        self.matchRequest()

    u"""-----------------------------
        ::pkg:: Shimehari.contexts.RequestContext
        matchRequest
        ~~~~~~~~~~~~

        リクエストに対してマッチするルールが存在するかチェックします

    ------------------------------"""
    def matchRequest(self):
        try:
            urlRule, self.request.viewArgs = self.urlAdapter.match(return_rule=True)
            self.request.urlRule = urlRule
        except HTTPException, e:
            self.request.routingException = e

    u"""-----------------------------
        ::pkg:: Shimehari.contexts.RequestContext
        push
        ~~~~

        現在のリクエストコンテキストをスタックに追加します。

    ------------------------------"""
    def push(self):
        top = _requestContextStack.top
        if top is not None and top.preserved:
            top.pop()

        self._pushedApplicationContext = _getNecessaryAppContext(self.app)
        _requestContextStack.push(self)

        self.session = self.app.openSession(self.request)
        if self.session is None:
            self.session = self.app.makeNullSession()

    u"""-----------------------------
        ::pkg:: Shimehari.contexts.RequestContext
        pop
        ~~~

        リクエストコンテキストをスタックから取り出します。

    ------------------------------"""
    def pop(self, exc=None):
        self.preserved = False

        if exc is None:
            exc = sys.exc_info()[1]
        self.app.doRequestContextTearDown(exc)
        rv = _requestContextStack.pop()

        assert rv is self, 'zibun jan'
        rv.request.environ['werkzeug.request'] = None

        if self._pushedApplicationContext:
            self._pushedApplicationContext.pop(exc)
            self._pushedApplicationContext = None

    def __enter__(self):
        self.push()
        return self

    def __exit__(self, exc_type, exc_value, tb):
        if self.request.environ.get('shimehari._preserve_context') or (tb is not None and self.app.preserveContextOnException):
            self.preserved = True
        else:
            self.pop(exc_value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
from optparse import make_option

from shimehari.core.exceptions import CommandError
from shimehari.core.manage import AbstractCommand
from shimehari.core.helpers import importFromString
from shimehari.configuration import ConfigManager


class Command(AbstractCommand):
    name = 'checkconfig'
    summary = 'Check Your Shimehari Application Configration'
    usage = "Usage: %prog [OPTIONS]"
    option_list = AbstractCommand.option_list + (
        make_option('--env', '-e', action='store', type='string', dest='environ', help='get config environ'),
    )

    def handle(self, *args, **options):
        try:
            importFromString('config')
        except ImportError:
            sys.path.append(os.getcwd())
            try:
                importFromString('config')
            except ImportError:
                raise CommandError(u'コンフィグファイルが見当たりません')

        env = options.get('env')
        if env is None:
            env = 'development'

        sys.stdout.write('\nYour Shimehari App Current Config.\n\n')
        sys.stdout.write('-------------------------------------------------\n')
        sys.stdout.write(ConfigManager.getConfig(env).dump())
        sys.stdout.write('\n')

Command()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    [[Shimehari Config]]
    config
    ~~~~~~

===============================
"""

from datetime import timedelta
from shimehari.configuration import Config, ConfigManager

ConfigManager.addConfigs([
    Config('development', {
        'DEBUG': False,

        'TEST': False,

        #アプリのディレクトリ名
        'APP_DIRECTORY': '%s',

        'MAIN_SCRIPT': 'main',

        #アプリケーションinstanceの名前
        'APP_INSTANCE_NAME': 'app',

        'CONTROLLER_DIRECTORY': 'controllers',

        'VIEW_DIRECTORY': 'views',

        'ASSESTS_DIRECTORY':'assets',

        #for daiginjou
        'MODEL_DIRECTORY': 'models',

        'PREFERRED_URL_SCHEME': 'http',

        'AUTO_SETUP': True,

        'CONTROLLER_AUTO_NAMESPACE': True,

        'TEMPLATE_ENGINE': 'jinja2',

        'SECRET_KEY': '_secret_shimehari',

        'SERVER_NAME': None,

        'PRESERVE_CONTEXT_ON_EXCEPTION': None,

        'PERMANENT_SESSION_LIFETIME': timedelta(days=31),

        #キャッシュ
        'CACHE_STORE': None,

        'CACHE_DEFAULT_TIMEOUT': 300,

        'CACHE_THRESHOLD': 500,

        'CACHE_KEY_PREFIX': None,

        'CACHE_DIR': None,

        'CACHE_OPTIONS': None,

        'CACHE_ARGS': [],

        'LOG_FILE_OUTPUT': False,

        'LOG_FILE_ROTATE': False,

        'LOG_ROTATE_MAX_BITE': (5 * 1024 * 1024),

        'LOG_ROTATE_COUNT': 5,

        'LOG_FILE_DIRECTORY': '%s/log',

        'LOG_DEBUG_FORMAT': '%s',

        'LOG_OUTPUT_FORMAT': '%s'
    })
])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    ::pkg:: Shimehari.core.command
    AbstractCommand
    ~~~~~~~~~~~~~~~
    コマンドラインツール

    抽象クラス
    Command の元となるクラス
===============================
"""

import sys
from optparse import OptionParser, make_option
import traceback
import shimehari

from shimehari.core.exceptions import CommandError

command_dict = {}


class AbstractCommand(object):
    name = None
    help = ''
    usage = 'Usage: shimehari COMMAND [ARGUMENTS...] [OPTIONS]'
    summary = 'Shimehari is a framework for Python'
    hidden = False
    option_list = (
        make_option('--traceback', action='store_true', help='Print traceback on exception'),
    )

    def __init__(self):
        self.parser = OptionParser(
            usage=self.usage,
            prog='shimehari %s' % self.name,
            version=self.getVersion(),
            option_list=self.option_list)
        if not self.name is None:
            command_dict[self.name] = self

    def getVersion(self):
        return shimehari.getVersion()

    def runFromArgv(self, argv):
        options, args = self.parser.parse_args(argv[2:])
        self.execute(*args, **options.__dict__)

    def execute(self, *args, **options):

        showTraceback = options.get('traceback', False)

        try:
            self.stdout = options.get('stdout', sys.stdout)
            self.stderror = options.get('stderr', sys.stderr)

            output = self.handle(*args, **options)
            if output:
                self.stdout.write(output)
        except (TypeError, CommandError), e:
            if showTraceback:
                traceback.print_exc()
            else:
                self.stderror.write('%s\n' % unicode(e).encode('utf-8'))
            sys.exit(1)

    def validate(self):
        raise NotImplementedError()

    def handle(self):
        raise NotImplementedError()

    #humu...
    def getAppConfig(self):
        pass


import os


class CreatableCommand(AbstractCommand):

    u"""-----------------------------
        ::pkg:: Shimehari.core.manage.AbstractCommand.CreatableCommand
        toWritable
        ~~~~~~~~~~

        ファイルを書き込み可能状態にします。
        [args]
            :filename 対象ファイル名
    ------------------------------"""
    def toWritable(self, filename):
        if sys.platform.startswith('java'):
            return

        if not os.access(filename, os.W_OK):
            st = os.stat(filename)
            # Undefined name "start"
            newPermission = stat.S_IMODE(st.st_mode)
            os.chmod(filename, newPermission)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from shimehari.shared import _requestContextStack
from shimehari.helpers import lockedCachedProperty
from werkzeug.datastructures import ImmutableDict


def _defaultTemplateCtxProcessor():
    reqctx = _requestContextStack.top
    return dict(
        config=reqctx.app.config,
        request=reqctx.request,
        session=reqctx.session,
        shared=reqctx.shared
    )


def _render(template, context):
    rv = template.render(context)
    return rv


def renderTemplate(templateNameOrList, **context):
    ctx = _requestContextStack.top
    ctx.app.updateTemplateContext(context)
    return _render(ctx.app.templateEnv.get_or_select_template(templateNameOrList),
        context)


def renderTempalteString(source, **context):
    ctx = _requestContextStack.top
    ctx.app.updateTemplateContext(context)
    return _render(ctx.app.templateEnv.from_string(source), context)


u"""
===============================
    ::pkg:: Shimehari.template
    Templater
    ~~~~~~~~~~

    各テンプレートエンジンに共通のインターフェースをもたらす
    アダプターのベーシッククラス

===============================
"""


class AbstractTemplater(object):
    templateOptions = ImmutableDict()

    def __init__(self, app, *args, **options):
        self.app = app

        if 'templateOptions' in options:
            self.templateOptions = options['templateOptions']

    @lockedCachedProperty
    def templateLoader(self):
        raise NotImplementedError()

    def templateEnv(self):
        raise NotImplementedError()

    def createTemplateEnvironment(self):
        raise NotImplementedError()

    def dispatchLoader(self, app):
        raise NotImplementedError()

    def updateTemplateContext(self, context):
        funcs = self.app.templateContextProcessors[None]
        org = context.copy()
        for func in funcs:
            context.update(func())
        context.update(org)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.testing
    ~~~~~~~~~~~~~~~~~
    テスト
===============================
"""

from contextlib import contextmanager
from werkzeug.test import Client, EnvironBuilder
from shimehari import _requestContextStack


def makeTestEnvironBuilder(app, path='/', baseUrl=None, *args, **kwargs):
    httpHost = app.config.get('SERVER_NAME')
    appRoot = app.config.get('APP_ROOT')
    if baseUrl is None:
        baseUrl = 'http://%s/' % (httpHost or 'localhost')
        if appRoot:
            baseUrl += appRoot.lstrip('/')
    return EnvironBuilder(path, baseUrl, *args, **kwargs)


class ShimehariClient(Client):

    preserveContext = False

    @contextmanager
    def sessionTransaction(self, *args, **kwargs):
        if self.cookieJar is None:
            raise RuntimeError('cookieeee')

        app = self.application
        environOverrides = kwargs.setdefault('environOverrides', {})
        self.cookie_jar.inject_wsgi(environOverrides)
        otherReqContext = _requestContextStack.top
        with app.testRequestContext(*args, **kwargs) as c:
            sess = app.openSession(c.request)
            if sess is None:
                raise RuntimeError('can not open session')
            _requestContextStack.push(otherReqContext)
            try:
                yield sess
            finally:
                _requestContextStack.pop()

            resp = app.responseClass()
            if not app.sessionStore.isNullSession(sess):
                app.saveSession(sess, resp)
            headers = resp.get_wsgi_headers(c.request.environ)
            self.cookie_jar.extract_wsgi(c.request.environ, headers)

    def open(self, *args, **kwargs):
        kwargs.setdefault('environ_overrides', {})['shimehari._preserve_context'] = self.preserveContext

        asTuple = kwargs.pop('asTuple', False)
        buffered = kwargs.pop('buffered', False)
        followRedirects = kwargs.pop('followRedirects', False)
        builder = makeTestEnvironBuilder(self.application, *args, **kwargs)

        return Client.open(self, builder,
                           as_tuple=asTuple,
                           buffered=buffered,
                           follow_redirects=followRedirects)

    def __enter__(self):
        if self.preserveContext:
            raise RuntimeError('can not nest')
        self.preserveContext = True
        return self

    def __exit__(self, excType, excValue, tb):
        self.preserveContext = False
        top = _requestContextStack.top
        if top is not None and top.preserved:
            top.pop()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.core.manage.commands
    drink
    ~~~~~

    Shimehari App を起動します。

===============================
"""

import os
from optparse import make_option

from shimehari.core.manage import AbstractCommand
from shimehari.core.helpers import importFromString
from shimehari.configuration import ConfigManager
from shimehari.core.exceptions import DrinkError, CommandError

DEFAULT_PORT = 5959
DEFAULT_HOST = '127.0.0.1'


class BaseDrinkCommand(AbstractCommand):
    name = 'drink'
    summary = 'Present a web page at http://127.0.0.1:5959/'
    usage = "Usage: %prog COMMAND [OPTIONS]"

    option_list = AbstractCommand.option_list + (
        make_option('--port', '-p', action='store', type='int', dest='port', help='port number.'),
        make_option('--host', action='store', type='string', dest='host', help='host name.'),
        make_option('--debug', '-d', action='store_true', default=False, help='running debug mode.'),
    )

    def __init__(self):
        super(BaseDrinkCommand, self).__init__()
        self.port = DEFAULT_PORT
        self.host = DEFAULT_HOST
        self.debug = False

    def run(self, *args, **options):
        try:
            #humu-
            import config
        except ImportError, e:
            import sys
            sys.path.append(os.getcwd())
            try:
                import config
            except:
                pass

        try:
            currentEnv = options.get('SHIMEHARI_ENV')
            currentConfig = ConfigManager.getConfig(currentEnv or 'development')
            app = importFromString(currentConfig['APP_DIRECTORY'] + '.' + currentConfig['MAIN_SCRIPT'] + '.' + currentConfig['APP_INSTANCE_NAME'])
            if not self.debug and currentConfig['DEBUG']:
                self.debug = True

            def openBrowser(host, port):
                url = 'http://'
                if not host:
                    url += '127.0.0.1'
                else:
                    url += host

                if not port:
                    url += ':5959/'
                else:
                    url += ':' + str(port)
                import webbrowser
                webbrowser.open(url)
            import threading
            timer = threading.Timer(0.5, openBrowser, args=[self.host, self.port])
            timer.start()
            app.drink(host=self.host, port=int(self.port), debug=self.debug)

        except Exception, e:
            raise CommandError(e)

    def handle(self, *args, **options):
        port = options.get('port')
        if port:
            self.port = port

        host = options.get('host')
        if not host:
            self.host = host
        self.debug = options.get('debug')
        self.run(*args, **options)

BaseDrinkCommand()


class Command(BaseDrinkCommand):
    def getHandler(self):
        pass



#!/usr/bin/env python
# -*- cofing: utf-8 -*-

from shimehari import Router, Resource
from controllers import IndexController

#
# You can routing like Rails style.
#
# from shimehari.routing import RESTfulRouter
# appRoutes = RESTfulRouter([
#    Resource(IndexController,root=True),
#    Resource(SomeOtherController),
#    Resource(SomethingElseController, expect=['edit', 'destroy'])
#])


appRoutes = Router([
    Resource(IndexController, root=True)
])

#
# You can routing like werkzeug style.
#
# from shimehari import BasicRouter
# appRoutes = BasicRouter([
#    {'/': IndexController.list},
#    {'/hoge': SomeOtherController.list},
#    {'/hoge/<int:id>': SomeOtherController.show}
#])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.core.manage.commands.routes
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    現在のルーティング状況をコマンドラインへダンプします。

===============================
"""

import os
import sys

from shimehari.core.exceptions import CommandError
from shimehari.core.manage import AbstractCommand
from shimehari.core.helpers import importFromString
from shimehari.configuration import ConfigManager, Config
from shimehari.helpers import getEnviron


class Command(AbstractCommand):
    name = 'routes'
    summary = "Show Application routing"
    usage = "Usage: %prog [OPTIONS]"

    def __init__(self):
        super(Command, self).__init__()

    def handle(self, *args, **options):
        try:
            importFromString('config')
        except ImportError:
            sys.path.append(os.getcwd())
            try:
                importFromString('config')
            except ImportError:
                raise CommandError(u'コンフィグファイルが見当たりません')

        config = ConfigManager.getConfig(getEnviron())
        appPath = os.path.join(os.getcwd(), config['APP_DIRECTORY'])
        if not os.path.isdir(appPath):
            raise CommandError(u'アプリケーションが見当たりません')

        try:
            router = importFromString(config['APP_DIRECTORY'] + '.' + 'router')
        except Exception, e:
            raise CommandError(u'ルーターがみつかりません。\n %s' % e)

        sys.stdout.write('\nYour Shimehari App Current Routing.\n\n')
        sys.stdout.write('Methods       |URL                          |Action\n')
        sys.stdout.write('----------------------------------------------------------------------\n')
        sys.stdout.write(router.appRoutes.dump())
        sys.stdout.write('\n')

Command()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from shimehari import Request, Response, ApplicationController


class IndexController(ApplicationController):
    def __init__(self, name):
        ApplicationController.__init__(self, name)

    #Create your controller here.

    def index(self, *args, **kwargs):
        return Response('Drink Shimehari!')

    def show(self, *args, **kwargs):
        return Response('write show action code...')

    def edit(self, *args, **kwargs):
        return Response('write edit action code...')

    def new(self, *args, **kwargs):
        return Response('write new action code...')

    def create(self, *args, **kwargs):
        return Response('write create action code...')

    def update(self, *args, **kwargs):
        return Response('write update action code...')

    def destroy(self, *args, **kwargs):
        return Response('write destroy action code...')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.wrappers
    ~~~~~~~~~~~~~~~~~~
    リクエストやらレスポンスやら

===============================
"""

from werkzeug.utils import cached_property
from werkzeug.wrappers import Request as RequestBase, Response as ResponseBase
from shimehari.core.helpers import attachEnctypeErrorMultidict
from shimehari.core.exceptions import JSONBadRequest
from shimehari.helpers import _assertHaveJson, json


class Request(RequestBase):
    urlRule = None
    viewArgs = None

    routingException = None

    @property
    def endpoint(self):
        if self.urlRule is not None:
            return self.urlRule.endpoint

    @cached_property
    def json(self):
        if __debug__:
            _assertHaveJson()
        if self.mimetype == 'application/json':
            reqCharset = self.mimetype_params.get('charset')
            try:
                if reqCharset is not None:
                    return json.loads(self.data, encoding=reqCharset)
                return json.loads(self.data)
            except ValueError, error:
                return self.jsonLoadFailedHandler(error)

    def jsonLoadFailedHandler(self, error):
        raise JSONBadRequest()

    def _load_from_data(self):
        RequestBase._load_from_data(self)

        # FIXME: Undefined name "_requestContextStack"
        context = _requestContextStack

        if context is not None and context.app.debug \
           and self.mimetype != 'multipart/form-data' and not self.files:
            attachEnctypeErrorMultidict(self)


class Response(ResponseBase):
    default_mimetype = 'text/html'



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class ShimehariException(Exception):
    def __init__(self, description):
        self.description = description

    def __str__(self):
        return 'ShimehariError: %s' % self.description


class ShimehariHTTPError(ShimehariException):
    def __init__(self, description, host='127.0.0.1', port=5959):
        ShimehariException.__init__(description)


class ShimehariSetupError(ShimehariException):
    def __str__(self):
        return 'ShimehariSetupError: %s' % self.description


class CommandError(Exception):
    pass


class DrinkError(ShimehariException):
    def __init__(self, description, host=None, port=None):
        super(ShimehariException).__init__(description)
        # self.description = description
        self.host = host
        self.port = port

    def __str__(self):
        return 'Shimehari Drink Command is Error: %s\n host=%s\n port=%d' % (self.description, self.host, self.port)

    # def __repr__(self):
        # return 'Shimehari Drink Command is Error: %s\n host=%s\n port=%d' % (self.description, self.host, self.port)


from werkzeug.exceptions import HTTPException, BadRequest
from shimehari.helpers import json


class JSONHTTPExceptions(HTTPException):
    def get_body(self, environ):
        return json.dumps(dict(description=self.get_description(environ)))

    def get_headers(self, environ):
        return [('Content-Type', 'application/json')]


class JSONBadRequest(JSONHTTPExceptions, BadRequest):
    description = ('ブラウザ、もしくはプロクシが送ったリクエストを、このアプリケーションでは処理できません。')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.cache
    ~~~~~~~~~~~~~~~~~~~~~~~

    キャッシュオブジェクト

===============================
"""

from functools import wraps

from shimehari.core.cachestore import SimpleCacheStore, MemcachedCacheStore, FileSystemCacheStore, RedisCacheStore, NullCacheStore
from shimehari.configuration import ConfigManager, Config
from shimehari.helpers import getEnviron
from shimehari.shared import request, currentApp

u"""
===============================
    Shimehari.cache.Cache
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    キャッシュ
===============================
"""


class Cache(object):
    store = None

    def __init__(self, storetype=None):
        config = ConfigManager.getConfig(getEnviron())

        if config['CACHE_STORE'] is not None:
            store = config['CACHE_STORE']
        else:
            store = storetype

        self.store = self._importCacheStore(store)

    u"""-----------------------------
        Shimehari.cache.Cache
        ~~~~~~~~~~~~~~~~~~~~~~~~
        get

        キャッシュストアから指定したキーに該当するキャッシュを取得します。
        [args]
            :keys   キー

        [return]
            :value  キャッシュ

    ------------------------------"""
    def get(self, *args, **kwargs):
        return self.store.get(*args, **kwargs)

    def getMany(self, *args, **kwargs):
        return self.store.getMany(*args, **kwargs)

    def getDict(self, *args, **kwargs):
        return self.store.getDict(*args, **kwargs)

    def set(self, *args, **kwargs):
        self.store.set(*args, **kwargs)

    def setMany(self, *args, **kwargs):
        self.store.setMany(*args, **kwargs)

    def add(self, *args, **kwargs):
        self.store.add(*args, **kwargs)

    def delete(self, *args, **kwargs):
        self.store.delete(*args, **kwargs)

    def deleteMany(self, *args, **kwargs):
        self.store.deleteMany(*args, **kwargs)

    def clear(self):
        self.store.clear()

    def setCacheStore(self, store):
        self.store = store

    #u-mu....
    def cached(self, timeout=None, key_prefix='view/%s', unless=None):
        def decorator(f):
            @wraps(f)
            def decoratedFunction(*args, **kwargs):
                if callable(unless) and unless() is True:
                    return f(*args, **kwargs)
                cacheKey = decoratedFunction.createCacheKey(*args, **kwargs)

                rv = self.store.get(cacheKey)
                if rv is None:
                    rv = f(*args, **kwargs)
                    self.store.set(cacheKey, rv, decoratedFunction.cacheTimeout)
                return rv

            def createCacheKey(*args, **kwargs):
                if callable(key_prefix):
                    cacheKey = key_prefix()
                elif '%s' in key_prefix:
                    cacheKey = key_prefix % request.path
                else:
                    cacheKey = key_prefix

                cacheKey = cacheKey.encode('utf-8')

                return cacheKey

            decoratedFunction.createCacheKey = createCacheKey
            decoratedFunction.cacheTimeout = timeout
            decoratedFunction.uncached = f

            return decoratedFunction
        return decorator

    def _importCacheStore(self, storetype):
        if storetype == 'simple':
            return SimpleCacheStore()
        if storetype == 'memcache':
            return MemcachedCacheStore()
        if storetype == 'file':
            config = ConfigManager.getConfig(getEnviron())
            return FileSystemCacheStore(cacheDir=config['CACHE_DIR'])
        if storetype == 'redis':
            return RedisCacheStore()
        return NullCacheStore()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.core
    helpers
    ~~~~~~~

    主に Shimehari 内部で使われることを
    想定した helper

===============================
"""

import sys


u"""--------------------------------------
    ::pkg:: Shimenari.core.helpers
    importFromString
    ~~~~~~~~~~~~~~~~

--------------------------------------"""


def importFromString(targetName):
    if isinstance(targetName, unicode):
        targetName = str(targetName)
    try:
        if '.' in targetName:
            pkgs, obj = targetName.rsplit('.', 1)
        else:
            return __import__('config')
        try:
            return getattr(__import__(pkgs, fromlist=[obj]), obj)
        except (ImportError, AttributeError):
            pkgName = pkgs + '.' + obj
            __import__(pkgName)
            return sys.modules[pkgName]

    except ImportError, error:
        raise ImportError(error)


class DebugFilesKeyError(KeyError, AssertionError):
    def __init__(self, request, key):
        formMatches = request.form.getlist(key)
        buf = ['aaaa %s %s' % (key, request.mimetype)]

        if formMatches:
            buf.append('\n\n %s' % ''.join('"%s"' % x for x in formMatches))
        self.msg = ''.join(buf).encode('utf-8')

    def __str__(self):
        return self.msg


def attachEnctypeErrorMultidict(request):
    oldCls = request.files.__class__

    class newCls(oldCls):
        def __getitem__(self, key):
            try:
                return oldCls.__getitem__(self, key)
            except KeyError, e:
                if key not in request.form:
                    raise
                raise DebugFilesKeyError(request, key)
    newCls.__name__ = oldCls.__name__
    newCls.__module__ = oldCls.__module__
    request.files.__class__ = newCls


def importPreferredMemcachedClient(servers):
    try:
        import pylibmc
    except ImportError, e:
        pass
    else:
        return pylibmc.Client(servers)

    try:
        import memcache
    except ImportError, e:
        try:
            import cmemcache as memcache
        except ImportError, e:
            pass
    else:
        return memcache.Client(servers)

    try:
        from google.appengine.api import memecache
    except ImportError, e:
        pass
    else:
        return memcache.Client()

    raise RuntimeError('no memcache modules.')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.shared
    ~~~~~~~~~~~~~~~~
    アプリケーション内で共通して使いたい物たち

===============================
"""
from functools import partial
from werkzeug.local import LocalProxy, LocalStack


class _SharedRequestClass(object):
    pass


def _lookUpObjectInRequestContext(name):
    top = _requestContextStack.top
    if top is None:
        raise RuntimeError('ahixi...')
    return getattr(top, name)


def findApp():
    top = _appContextStack.top
    if top is None:
        raise RuntimeError('AppContext is none...')
    return top.app


_requestContextStack = LocalStack()
_appContextStack = LocalStack()
currentApp = LocalProxy(findApp)
request = LocalProxy(partial(_lookUpObjectInRequestContext, 'request'))
session = LocalProxy(partial(_lookUpObjectInRequestContext, 'session'))
shared = LocalProxy(partial(_lookUpObjectInRequestContext, 'shared'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

signalsAvailable = False
try:
    from blinker import Namespace
    signalsAvailable = True
except ImportError:
    class Namespace(object):
        def signal(self, name, doc=None):
            return _DummySignal(name, doc)
        pass

    class _DummySignal(object):
        def __init__(self, name, doc=None):
            self.name = name
            self.doc = doc

        def _fail(self, *args, **options):
            raise RuntimeError('ooooo')

        send = lambda *a, **kw: None
        connect = disconnect = has_recivers_for = recivers_for = temporaily_connected_to = connected_to = _fail
        del _fail


_signals = Namespace()

requestContextTearingDown = _signals.signal('requestcontext-tearing-down')
appContextTearingDown = _signals.signal('appcontext-tearing-down')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
u"""
===============================
    Shimehari.app
    ~~~~~~~~~~~~~
    Flask などなど参考に。
    ルーティングが多少複雑になっても
    対応できるような作りにしたいなぁ
===============================
"""

import os
import sys
from  threading import Lock, RLock
from functools import update_wrapper
from datetime import timedelta

from werkzeug.exceptions import NotFound, HTTPException, InternalServerError, MethodNotAllowed, BadRequest
from werkzeug.datastructures import ImmutableDict
from werkzeug.routing import RequestRedirect, Rule

from .helpers import _Kouzi, findPackage, getHandlerAction, getModulesFromPyFile, getEnviron, \
                    lockedCachedProperty, getTemplater
from .contexts import RequestContext, AppContext
from .routing import Router
from core.config import RESTFUL_ACTIONS
from .wrappers import Request, Response
from shimehari.configuration import Config, ConfigManager
from shimehari.session import SessionStore
from shimehari.shared import _requestContextStack, _appContextStack, _SharedRequestClass, request
from shimehari.template import _defaultTemplateCtxProcessor
from shimehari.core.exceptions import ShimehariException, ShimehariSetupError
from shimehari.core.signals import appContextTearingDown, requestContextTearingDown


_loggerLock = Lock()


def setupMethod(f):
    def wrapperFunc(self, *args, **kwargs):
        if self.debug or self._gotFirstRequest:
            raise AssertionError('Setup seems to have already completed ...')
        return f(self, *args, **kwargs)
    return update_wrapper(wrapperFunc, f)


class Shimehari(_Kouzi):
    u"""Shimehari Object は WSGI アプリケーションとして必要な機能を実装しており、
    アプリケーションの中心となるオブジェクトです。

    メインモジュール、または __init__.py ファイルの中で以下のように書くことで
    Shimehari インスタンスを生成することができます。

    .. code-block:: python

        from shimehari import Shimehari
        app = Shimehari(__name__)

    ただし、Shimehari では通常コマンドラインでアプリケーションを生成することを推奨しているので
    あなたが直接インスタンス生成のコードを書くことはそうそうないと思われます。

    :param importName: アプリケーションのパッケージ名
    :param staticURL: サイト内共通、静的ファイルの URL
    :param staticFolder: 静的ファイルが格納されているディレクトリ
    :param appFolder: アプリケーション全体が格納されているディレクトリ
    :param controllerFolder: コントローラーが格納されているディレクトリ
    :param viewFolder: ビューが格納されているディレクトリ
    :param assetsFolder: アセットファイルが格納されているディレクトリ
    :param instancePath: アプリケーションのための代替インスタンスパス

    """

    currentEnv = getEnviron()
    debug = None
    testing = None

    requestClass = Request
    responseClass = Response

    testClientCls = None

    teardownAppContextFuncs = []

    defaultConfig = {
        'DEBUG': False,
        'TEST': False,
        'APP_DIRECTORY': 'app',
        'CONTROLLER_DIRECTORY': 'controllers',
        'VIEW_DIRECTORY': 'views',
        #for daiginjou
        'MODEL_DIRECTORY': 'models',
        'PREFERRED_URL_SCHEME': 'http',
        'AUTO_SETUP': True,
        'TEMPLATE_ENGINE': 'jinja2',
        'TRAP_HTTP_EXCEPTIONS': False,
        'SERVER_NAME': None,
        'PERMANENT_SESSION_LIFETIME': timedelta(days=31),
        'SECRET_KEY': '_secret_shimehari'
    }

    templateOptions = {}

    sessionStore = SessionStore()

    sharedRequestClass = _SharedRequestClass

    def __init__(self, importName,
                 staticURL=None, staticFolder='static',
                 appFolder='app', controllerFolder='controllers',
                 viewFolder='views', assetsFolder='assets',
                 instancePath=None, isRelativeConfig=False, templateOptions={}):

        _Kouzi.__init__(self, importName, appFolder=appFolder,
                        controllerFolder=controllerFolder, viewFolder=viewFolder)

        if instancePath is None:
            self._instancePath = self.getInstancePath()

        self._logger = None
        self.loggerName = self.importName

        self.config = self.getConfig()

        self.controllers = {}
        self.urlValuePreprocesors = {}
        self.beforeRequestFuncs = {}
        self.beforeFirstRequestFuncs = []
        self.urlDefaultFuncs = {}
        self.afterRequestFuncs = {}
        self._errorHandlers = {}
        self.errorHandlerSpec = {None: self._errorHandlers}
        self.buildErrorHandlers = None
        self.teardownRequestContextFuncs = {}

        self.templateContextProcessors = {
            None: [_defaultTemplateCtxProcessor]
        }

        #CSRF
        from shimehari.crypt import CSRF
        self.csrf = CSRF(self)

        self._router = Router()

        self._gotFirstRequest = False

        self._beforeRequestLock = Lock()

        self.debug = self.config['DEBUG']
        self.test = self.config['TEST']
        self.sessionKey = self.config['SECRET_KEY']
        self.useXSendFile = self.config['USE_X_SENDFILE']

        self.templateOptions = templateOptions

        if self.config['AUTO_SETUP']:
            self.setup()

    @property
    def gotFirstRequest(self):
        return self._gotFirstRequest

    @property
    def propagateExceptions(self):
        return self.testing or self.debug

    @property
    def preserveContextOnException(self):
        rv = self.config['PRESERVE_CONTEXT_ON_EXCEPTION']
        if rv is not None:
            return rv
        return self.debug

    @property
    def logger(self):
        if self._logger and self._logger.name == self.loggerName:
            return self._logger
        with _loggerLock:
            if self._logger and self._logger.name == self.loggerName:
                return self._logger
            from shimehari.logging import createLogger
            self._logger = rv = createLogger(self.loggerName)
            return rv

    def router():
        u"""アプリケーションのルーティングを管理するルーターを設定します。"""
        def fget(self):
            return self._router

        def fset(self, value):
            self.setControllerFromRouter(value)
            self._router = value

        def fdel(self):
            self.controllers = {}
            del self._router
        return locals()
    router = property(**router())

    def getInstancePath(self):
        u"""インスタンスパスを返します。"""
        prefix, pkgPath = findPackage(self.importName)
        if prefix is None:
            return os.path.join(pkgPath, 'instance')
        return os.path.join(prefix, 'var', self.name + '-instance')

    def getConfig(self):
        u"""現在アプリケーションに適用されているコンフィグを返します。"""
        configs = ConfigManager.getConfigs()
        try:
            # from .config import config
            configs = ConfigManager.getConfigs()
        except ImportError:
            pass
        if not configs:
            cfg = Config(self.currentEnv, self.defaultConfig)
            ConfigManager.addConfig(cfg)
            return cfg
        else:
            return configs[self.currentEnv]

    def saveSession(self, session, response):
        u"""セッションを保存します。

        :param session:     保存したいセッション
        :param response:    レスポンス

        """
        if session.should_save:
            self.sessionStore.save(session, response)
            response.set_cookie(self.sessionKey, session.sid)
        return response

    def openSession(self, request):
        sid = request.cookies.get(self.sessionKey, None)
        if sid is None:
            return self.sessionStore.new()
        else:
            return self.sessionStore.get(sid)

    def setControllerFromRouter(self, router):
        u"""設定されたルーターからコントローラーをバインディングします。

        :param router: ルーター
        """
        if not self.controllers:
            self.controllers = {}
        for rule in router._rules:
            self.controllers[rule.endpoint] = rule.endpoint

    def addController(self, controller):
        u"""アプリケーションにコントローラーを追加します。

        :param controller:  追加したいコントローラー。
                            追加されたコントローラーはアプリケーションの管理下に置かれ、
                            ルーティングが自動生成されます。
        """
        for action in RESTFUL_ACTIONS:
            handler = getHandlerAction(controller, action)
            if handler is not None:
                self.controllers[handler] = handler

    def addRoute(self, url, func, methods=None, **options):
        rule = Rule(url, endpoint=func.__name__, methods=methods)
        self.controllers[func.__name__] = func
        self.router.add(rule)

    def logException(self, excInfo):
        self.logger.error('excepts on %s [%s]' % (request.path, request.method), exc_info=excInfo)

    def injectURLDefaults(self, endpoint, values):
        funcs = self.urlDefaultFuncs.get(None, ())
        for func in funcs:
            func(endpoint, values)

    @lockedCachedProperty
    def templateLoader(self):
        return self.templater.templateLoader

    @lockedCachedProperty
    def templateEnv(self):
        rv = self.templater.templateEnv()
        return rv

    def createTemplateEnvironment(self):
        rv = self.templater.createTemplateEnvironment()
        return rv

    def createGlobalTemplateLoader(self):
        return self.templater.dispatchLoader(self)

    def updateTemplateContext(self, context):
        self.templater.updateTemplateContext(context)

    def setup(self):
        u"""アプリケーションをセットアップします。
            指定された app ディレクトリ配下にあるコントローラー、ルーターを探し出しバインドします。
        """
        self.appPath = os.path.join(self.rootPath, self.appFolder)
        if not os.path.isdir(self.appPath):
            raise ShimehariSetupError('Application directory is not found\n%s' % self.rootPath)
            sys.exit(0)
        try:
            __import__(self.appFolder)
            self.setupTemplater()
            self.setupBindController()
            self.setupBindRouter()
        except (ImportError, AttributeError):
            raise ShimehariSetupError('Application directory is invalid')

    def setupTemplater(self):
        try:
            self.templater = getTemplater(self, self.config['TEMPLATE_ENGINE'], templateOptions=self.templateOptions)
        except Exception, e:
            raise ShimehariSetupError('setup template engine was failed... \n%s' % e)

    def setupBindController(self):
        u"""コントローラーをバインドします"""
        self.controllerPath = os.path.join(self.appPath, self.controllerFolder)
        if not os.path.isdir(self.controllerPath):
            raise ShimehariSetupError('Controller in the specified directory does not exist. %s' % self.controllerPath)

        try:
            ctrlDir = self.appFolder + '.' + self.controllerFolder
            __import__(ctrlDir)
            getModulesFromPyFile(self.controllerPath, self.rootPath)
        except (ImportError, AttributeError), error:
            raise ShimehariSetupError('setup controller was failed... \n%s' % error)

    def setupBindRouter(self):
        u"""ルーターをバインドします。"""
        try:
            routerFile = self.appFolder + '.' + 'router'
            routerMod = __import__(routerFile, fromlist=['router'])
            if hasattr(routerMod, 'appRoutes'):
                self.router = routerMod.appRoutes
            if self.hasStaticFolder:
                self.addRoute(self.staticURL + '/<path:filename>', self.sendStaticFile)
        except (ImportError, AttributeError), e:
            raise ShimehariSetupError('Failed to setup the router ...\n details::\n%s' % e)

    @setupMethod
    def beforeRequest(self, f):
        u"""リクエストを処理する前に実行したいメソッドを登録します。

        :param f: リクエスト処理前に実行させたい処理
        """
        self.beforeRequestFuncs.setdefault(None, []).append(f)
        return f

    @setupMethod
    def beforeFirstRequest(self, f):
        u"""アプリケーションに対し初めてリクエストがあったときのみ、
            リクエストを処理する前に実行したいメソッドを登録します。

        :param f: 初めてのリクエスト処理前に実行したい処理
        """
        self.beforeFirstRequestFuncs.append(f)

    @setupMethod
    def afterRequest(self, f):
        u"""リクエスト処理が終わった後に実行したいメソッドを登録します。

        :param f: リクエスト処理後に実行したい処理
        """
        self.afterRequestFuncs.setdefault(None, []).append(f)
        return f

    @setupMethod
    def urlValuePreprocessor(self, f):
        u"""リクエストの前に実行したい処理を登録します。

        :param f: 実行したい処理
        """
        self.urlValuePreprocesors.setdefault(None, []).append(f)
        return f

    @setupMethod
    def tearDownAppContext(self, f):
        self.teardownAppContextFuncs.append(f)
        return f

    @setupMethod
    def errorHandler(self, codeOrException):
        def decorator(f):
            self._registerErrorHandler(None, codeOrException, f)
            return f
        return decorator

    @setupMethod
    def _registerErrorHandler(self, key, codeOrException, f):
        if isinstance(codeOrException, HTTPException):
            codeOrException = codeOrException.code
        if isinstance(codeOrException, (int, long)):
            assert codeOrException != 500 or key is None
            self.errorHandlerSpec.setdefault(key, {})[codeOrException] = f
        else:
            self.errorHandlerSpec.setdefault(key, {}).setdefault(None, []).append((codeOrException, f))

    def tryTriggerBeforeFirstRequest(self):
        u"""アプリケーションに対し最初のリクエストがあった時のみに行う処理を
            実際に実行します。
        """
        if self._gotFirstRequest:
            return
        with self._beforeRequestLock:
            if self._gotFirstRequest:
                return
            self._gotFirstRequest = True
            [f() for f in self.beforeFirstRequestFuncs]

    def createAdapter(self, request):
        u"""url adapter を生成します

        :param request: 元となるリクエストオブジェクト


        """
        if request is not None:
            return self.router.bind_to_environ(request.environ)
        #なんのこっちゃ
        if self.config['SERVER_NAME'] is not None:
            return self.router.bind(
                self.config['SERVER_NAME'],
                script_name=self.config['APP_ROOT'] or '/',
                url_scheme=self.config['PREFERRED_URL_SCHEME'])

    def appContext(self):
        u"""アプリケーションコンテキストを返します。"""
        return AppContext(self)

    def requestContext(self, environ):
        u"""リクエストコンテキストを返します。

        :param environ: リクエスト環境変数
        """
        return RequestContext(self, environ)

    def doAppContextTearDonw(self, exc=None):
        if exc is None:
            exc = sys.exc_info()[1]
        for func in reversed(self.teardownAppContextFuncs):
            func(exc)
        appContextTearingDown.send(self, exc)

    def doRequestContextTearDown(self, exc=None):
        if exc is None:
            exc = sys.exc_info()[1]
        for func in reversed(self.teardownRequestContextFuncs.get(None, ())):
            func(exc)
        requestContextTearingDown.send(self, exc=exc)

    def preprocessRequest(self):
        u"""リクエスト前に処理したい登録済みのメソッドを実行します。

        :param rv: 登録した処理
        """
        for func in self.urlValuePreprocesors.get(None, ()):
            func(request.endpoint, request.viewArgs)
        self.csrf.checkCSRFExempt()
        self.csrf.csrfProtect()
        for func in self.beforeRequestFuncs.get(None, ()):
            rv = func()
            if rv is not None:
                return rv

    def processResponse(self, response):
        u"""レスポンスを返す前に処理したい登録済みのメソッドをを実行します。

        :param response: レスポンス
        """
        context = _requestContextStack.top
        funcs = ()

        if None in self.afterRequestFuncs:
            funcs = self.afterRequestFuncs[None]
        for handler in funcs:
            response = handler(response)

        if not self.sessionStore.isNullSession(context.session):
            self.saveSession(context.session, response)
        return response

    def dispatchRequest(self):
        u"""リクエストをもとに、レスポンスを発行します。

        :param request: リクエスト

        """
        self.tryTriggerBeforeFirstRequest()
        try:
            rv = self.preprocessRequest()
            if rv is None:
                req = _requestContextStack.top.request
                if req.routingException is not None:
                    self.raiseRoutingException(req)
                rule = req.urlRule
                rv = self.controllers[rule.endpoint](**req.viewArgs)
        except Exception, e:
            rv = self.makeResponse(self.handleUserException(e))

        response = self.makeResponse(rv)
        response = self.processResponse(response)

        return response

    def makeResponse(self, rv):
        u"""レスポンスを生成して返します。

        :param rv: リクエスト

        """
        status = headers = None

        if isinstance(rv, tuple):
            rv, status, headers = rv + (None,) * (3 - len(rv))

        if rv is None:
            raise ValueError('view function does not return a response.')

        if not isinstance(rv, self.responseClass):
            if isinstance(rv, basestring):
                rv = self.responseClass(rv, headers=headers, status=status)
                headers = status = None
            else:
                rv = self.responseClass.force_type(rv, request.environ)
        if status is not None:
            if isinstance(status, basestring):
                rv.status = status
            else:
                rv.status_code = status
        if headers:
            rv.headers.extend(headers)
        return rv

    def handleException(self, e):
        u"""エラーをハンドリングします。

        :param e: エラー内容

        """

        excType, excValue, excTb = sys.exc_info()
        handler = self.errorHandlerSpec[None].get(500)

        if self.propagateExceptions:
            if excValue is e:
                raise excType, excValue, excTb
            else:
                raise e

        self.logException((excType, excValue, excTb))
        if handler is None:
            return InternalServerError()
        return handler(e)

    def handleHTTPException(self, e):
        handler = self.errorHandlerSpec[None].get(e.code)
        if handler is None:
            return e
        return handler(e)

    def trapHTTPException(self, e):
        if self.config['TRAP_HTTP_EXCEPTIONS']:
            return True
        if self.config['TRAP_BAD_REQUEST_ERRORS']:
            return isinstance(e, BadRequest)
        return False

    def handleUserException(self, e):
        excType, excValue, tb = sys.exc_info()
        assert excValue is e

        if isinstance(e, HTTPException) and not self.trapHTTPException(e):
            return self.handleHTTPException(e)

        appHandlers = self.errorHandlerSpec[None].get(None, ())
        for typecheck, handler in appHandlers:
            if isinstance(e, typecheck):
                return handler(e)
        raise excType, excValue, tb

    def raiseRoutingException(self, request):
        if not self.debug or not isinstance(request.routingException, RequestRedirect) \
           or request.method in ('GET', 'HEAD', 'OPTIONS'):
            raise request.routingException

    def testRequestContext(self, *args, **kwargs):
        from shimehari.testing import makeTestEnvironBuilder
        builder = makeTestEnvironBuilder(self, *args, **kwargs)
        try:
            return self.requestContext(builder.get_environ())
        finally:
            builder.close()

    def handleBuildError(self, error, endpoint, **kwargs):
        if self.buildErrorHandlers is None:
            excType, excValue, tb = sys.exc_info()
            if excValue is error:
                raise excType, excValue, tb
            else:
                raise error
        return self.buildErrorHandlers(error, endpoint, **kwargs)

    def wsgiApp(self, environ, startResponse):
        u"""WSGI アプリとして実行します。(であってるのか)

        :param environ:         環境変数
        :param startResponse:   hoge

        """
        with self.requestContext(environ):
            try:
                response = self.dispatchRequest()
            except Exception, e:
                response = self.makeResponse(self.handleException(e))
            return response(environ, startResponse)

    def drink(self, host=None, port=None, debug=None, **options):
        u"""アプリを実行します。

        :param host:    ホスト名
        :param port:    ポート番号
        :param debug:   デバッグモードとして起動するかどうか
        :param options: kwargs
        """
        from werkzeug.serving import run_simple
        if host is None:
            host = '127.0.0.1'
        if port is None:
            port = 5959
        if debug is not None:
            self.debug = bool(debug)
        options.setdefault('use_reloader', self.debug)
        options.setdefault('use_debugger', self.debug)
        try:
            from werkzeug._internal import _log
            _log('info', ' * Shimehari GKGK!')
            run_simple(host, port, self, **options)
        finally:
            self._gotFirstRequest = False

    def run(self, host=None, port=None, debug=None, **options):
        u"""アプリを実行します。
            単純に drink メソッドをラップしているだけです。
            WSGI 周りのライブラリや既存のコードで run を自動的に呼ぶ物がおおいので念のため。
        """
        self.drink(host, port, debug, options)

    def testClient(self, useCookies=True):
        cls = self.testClientCls
        if cls is None:
            from shimehari.testing import ShimehariClient as cls
        return cls(self, self.responseClass, use_cookies=useCookies)

    def __call__(self, environ, startResponse):
        return self.wsgiApp(environ, startResponse)

    def __str__(self):
        return 'Shimehari WSGI Application Framework!'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import Command, setup
from shimehari import VERSION, AUTHOR


class runAudit(Command):
    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        pass

setup(
    name='Shimehari',
    version=VERSION,
    url='http://shimehari.hageee.net',
    license='BSD',
    author=AUTHOR,
    author_email='megane@glasses-factory.net',
    description='moderate framework for Python. based on Werkzeug, Jinja2',
    packages=['shimehari', 'shimehari.core', 'shimehari.template', 'shimehari.core.conf', 'shimehari.core.conf.app_template',
                'shimehari.core.conf.app_template.controllers', 'shimehari.core.conf.app_template.models',
                'shimehari.template.jinja2',
                'shimehari.core.manage', 'shimehari.core.manage.commands'],
    scripts=['shimehari/core/conf/shimehari'],
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=[
        'Werkzeug>=0.7',
        'Jinja2>=2.4'
    ],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    cmdclass={'audit': runAudit},
    test_suite='shimehari.testsuite.suite'
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.controllers
    ~~~~~~~~~~~~~~~~~~~~~
    コントローラーたち
===============================
"""
from abc import ABCMeta, abstractmethod

u"""
===============================
    Shimehari.controllers.AbstractController
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    コントローラーの抽象クラス
===============================
"""


class AbstractController(object):

    __metaclass__ = ABCMeta

    baseName = ''

    folderPrefix = ''

    def __init__(self, name):
        if not name:
            name = self.__class__.__name__
        self.name = name
        if 'Controller' in name:
            name = name.replace('Controller', '')

        self.baseName = name.lower()

    @abstractmethod
    def httpMethodNotAllowed(self, templateName='405.html'):
        return

    def bindHelper(self, helperName=None):
        pass


u"""
===============================
    Shimehari.controllers.TemplateRenderableMixIn
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    テンプレートレンダラブルみっくすいん
===============================
"""


class TemplateRenderableMixIn(object):

    def renderToResponse(self, context, **resKwargs):

        return

    def getTemplateName(self):
        if self.templateName is None:
            raise  # なんちゃかえらー
        else:
            return self.templateName


from shimehari.template import renderTemplate

u"""
===============================
    Shimehari.controllers.ApplicationController
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    コントローラーの具象クラス
===============================
"""


class ApplicationController(TemplateRenderableMixIn, AbstractController):

    _cache = None

    def cache():
        "The cache property."
        def fget(self):
            return self._cache

        #setter 塞ぐか
        def fset(self, value):
            self._cache = value

        def fdel(self):
            del self._cache
        return locals()
    cache = property(**cache())

    def __init__(self, name):
        AbstractController.__init__(self, name)

    def renderTemplate(self, templateName):
        return renderTemplate(templateName)

    def httpMethodNotAllowed(self, templateName='405.html'):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import posixpath
from jinja2 import BaseLoader, Environment as BaseEnvironment, TemplateNotFound, FileSystemLoader
from shimehari.helpers import lockedCachedProperty
from shimehari.template import AbstractTemplater
from shimehari.helpers import urlFor, _toJsonFilter
from shimehari.crypt import generateCSRFToken


class Environment(BaseEnvironment):
    def __init__(self, app, *args, **options):
        if 'loader' not in options:
            options['loader'] = app.createGlobalTemplateLoader()

        BaseEnvironment.__init__(self, *args, **options)
        self.app = app


class DispatchJinjaLoader(BaseLoader):
    def __init__(self, app):
        self.app = app

    def get_source(self, environment, template):
        for loader, localName in self._iterLoaders(template):
            try:
                return loader.get_source(environment, localName)
            except TemplateNotFound:
                pass
        raise TemplateNotFound(template)

    def _iterLoaders(self, template):
        loader = self.app.templateLoader
        if loader is not None:
            yield loader, template

    def listTemplates(self):
        result = set()
        loader = self.app.templateLoader
        if loader is not None:
            result.update(loader.list_templates())
        return list(result)


class Jinja2Templater(AbstractTemplater):
    def __init__(self, app, *args, **options):
        AbstractTemplater.__init__(self, app, *args, **options)

    @lockedCachedProperty
    def templateLoader(self):
        if self.app.viewFolder is not None:
            return FileSystemLoader(os.path.join(self.app.rootPath, self.app.appFolder, self.app.viewFolder))

    def templateEnv(self):
        rv = self.createTemplateEnvironment()
        return rv

    def createTemplateEnvironment(self):
        options = self.templateOptions

        if 'autoescape' in options:
            options['autoescape'] = self.selectJinjaAutoescape

        rv = Environment(self.app, **options)
        #いるかなー
        rv.globals.update(
            url_for=urlFor,
            csrfToken=generateCSRFToken
        )
        rv.filters['tojson'] = _toJsonFilter
        return rv

    def dispatchLoader(self, app):
        return DispatchJinjaLoader(app)

templater = Jinja2Templater



#!/usr/bin/env python
# -*- coding: utf-8 -*-


from werkzeug.exceptions import abort
from werkzeug.utils import redirect

from .app import Shimehari
from .controllers import ApplicationController
from .helpers import jsonAvailable, json, urlFor, jsonify, sendFile
from .routing import Router, Resource, RESTfulRule, Root, Group
from .shared import request, session, shared, _requestContextStack, _appContextStack, currentApp
from .template import renderTemplate, renderTempalteString
from .wrappers import Request, Response
from .cache import Cache

VERSION = '0.1.6'
AUTHOR = 'Yamaguchi Eikichi, Keiichiro Matsumoto'


def getVersion():
    return VERSION



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.core.manager
    ~~~~~~~~~~~~~~~~~~~~~~
    マネージメント
===============================
"""

import os
import sys
from optparse import OptionParser

from .AbstractCommand import AbstractCommand, CreatableCommand
from shimehari.core.helpers import importFromString

_commands = None


u"""
コマンドラインから色々実行するべさ
"""


def executeFromCommandLine(argv=None):
    executer = CommandLineExecuter(argv)
    executer.execute()


def loadCommandModule(cmdName, name):
    module = importFromString('%s.manage.commands.%s' % (cmdName, name))
    return module.Command()


def getCommands():
    global _commands
    if _commands is None:
        _commands = dict([(name, 'shimehari.core') for name in findCommand(__path__[0])])

        #ユーザーコマンドー…

    return _commands


def findCommand(manageDir):
    cmdDir = os.path.join(manageDir, 'commands')
    try:
        return [f[:-3] for f in os.listdir(cmdDir) if not f.startswith('_') and f.endswith('.py')]
    except OSError:
        return []


class CommandLineExecuter(object):
    def __init__(self, argv):
        self.argv = argv or sys.argv[:]
        self.progName = os.path.basename(self.argv[0])

    def fetchCommand(self, subcommand):
        try:
            cmdName = getCommands()[subcommand]
        except KeyError:
            sys.stdout.write("Unkown command: %r\nType %s help for usage. \n" % (subcommand, self.progName))
            sys.exit(1)
        if isinstance(cmdName, AbstractCommand):
            cls = cmdName
        else:
            cls = loadCommandModule(cmdName, subcommand)
        return cls

    def execute(self):
        parser = OptionParser()
        try:
            subcommand = self.argv[1]
        except:
            subcommand = 'help'

        if subcommand == 'version':
            from shimehari import getVersion
            sys.stdout.write('shimehari version: ' + getVersion() + '\n')
        elif self.argv[1:] == ['--version']:
            sys.stdout.write('shimehari version: ' + getVersion() + '\n')
        elif self.argv[1:] in (['--help'], ['-h']):
            self.fetchCommand('help').runFromArgv(self.argv)
        else:
            self.fetchCommand(subcommand).runFromArgv(self.argv)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.core.config
    ~~~~~~~~~~~~~~~~~~~~~
    shimehari core config settings
===============================
"""

from werkzeug.datastructures import ImmutableDict

u"""
    許可する RESTful アクション
"""
RESTFUL_ACTIONS = set(['index', 'show', 'edit', 'new', 'create', 'update', 'destroy'])

u"""
    許可する HTTP メソッド
"""
ALLOWED_HTTP_METHOD_NAMES = set(['get', 'post', 'put', 'delete', 'head', 'options', 'trace'])

u"""
    RESTful アクションごとに許可する HTTP メソッドの対応マップ
"""
RESTFUL_METHODS_MAP = ImmutableDict({
    'index': ['get'],
    'show': ['get', 'post'],
    'edit': ['get', 'post'],
    'new': ['get'],
    'create': ['post'],
    'update': ['put'],
    'destroy': ['delete']
})



# -*- coding: utf-8 -*-
#!/usr/bin/env python

u"""
===============================
    Shimehari.logging
    ~~~~~~~~~~~~~~~~~
    ロガー
    Flask などなど参考に。
    ルーティングが多少複雑になっても
    対応できるような作りにしたいなぁ
===============================
"""

from __future__ import absolute_import

import os
import sys

from logging import getLogger, StreamHandler, Formatter, getLoggerClass, DEBUG
from shimehari.configuration import ConfigManager, Config
from shimehari.helpers import getEnviron


def createLogger(loggerName='shimehariLoagger'):

    Logger = getLoggerClass()

    class DebugLogger(Logger):
        def getEffectiveLevel(x):
            if x.level == 0 and config['DEBUG']:
                return DEBUG
            return Logger.getEffectiveLevel(x)

    class DebugHandler(StreamHandler):
        def emit(x, record):
            StreamHandler.emit(x, record) if config['DEBUG'] else None
    config = ConfigManager.getConfig(getEnviron())
    if config['LOG_FILE_OUTPUT']:
        fn = os.path.join(config['LOG_FILE_DIRECTORY'], getEnviron()) + '.log'
        if config['LOG_FILE_ROTATE']:
            from logging import RotateFileHandler
            handler = RotateFileHandler(fn, 'a', config['LOG_ROTATE_MAX_BITE'], config['LOG_ROTATE_COUNT'])
        else:
            from logging import FileHandler
            handler = FileHandler(fn, 'a')

        handler.setFormatter(Formatter(config['LOG_OUTPUT_FORMAT']))
        logger = getLogger()
    else:
        handler = DebugHandler()
        handler.setFormatter(Formatter(config['LOG_DEBUG_FORMAT']))
        logger = getLogger(loggerName)

    logger.setLevel(DEBUG)

    del logger.handlers[:]
    logger.__class__ = DebugLogger
    logger.addHandler(handler)
    return logger



"""#!/usr/bin/env python
# -*- coding: utf-8 -*-

from shimehari import Request, Response, ApplicationController

class %sController(ApplicationController):
    def __init__(self, name):
        ApplicationController.__init__(self,name)

    #Create your controller here.

    def index(self, *args, **kwargs):
        pass

    def show(self, *args, **kwargs):
        pass

    def edit(self, *args, **kwargs):
        pass

    def new(self, *args, **kwargs):
        pass

    def create(self, *args, **kwargs):
        pass

    def update(self, *args, **kwargs):
        pass

    def destroy(self, *args, **kwargs):
        pass

"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.core.cache
    ~~~~~~~~~~~~~~~~~~~~~~~

    キャッシュストア

===============================
"""


import os
try:
    from hashlib import md5
except ImportError, e:
    from md5 import new as md5
import tempfile

from itertools import izip
from time import time


try:
    import cPickle as msg
except ImportError, e:
    import pickle as msg


def _items(mappp):
    return mappp.iteritems() if hasattr(mappp, 'iteritems') else mappp


u"""-----------------------------
    Shimehari.core.cachestore.BaseCacheStore
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    キャッシュストアの抽象クラス

------------------------------"""


class BaseCacheStore(object):

    def __init__(self, default_timeout=300):
        self.default_timeout = default_timeout

    u"""-----------------------------
        Shimehari.core.cachestore.BaseCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        get

        キャッシュストアから指定されたキーの値を取得します。
        [args]
            :key　   取得したいキャッシュのキー

    ------------------------------"""
    def get(self, key):
        pass

    u"""-----------------------------
        Shimehari.core.cachestore.BaseCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        getMany

        複数のキーからまとめてキャッシュを取得します。
        [args]
            :kyes   取得したいキャッシュのキー

    ------------------------------"""
    def getMany(self, *keys):
        return map(self.get, keys)

    u"""-----------------------------
        Shimehari.core.cachestore.BaseCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        clear

        複数のキーからまとめてキャッシュを dict として取得します。
        [args]
            :key

    ------------------------------"""
    def getDict(self, *keys):
        return dict(izip(keys, self.getMany(*keys)))

    u"""-----------------------------
        Shimehari.core.cachestore.BaseCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        delete

        指定されたキーをもつキャッシュを削除します。
        [args]
            :key

    ------------------------------"""
    def delete(self, key):
        pass

    u"""-----------------------------
        Shimehari.core.cachestore.BaseCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        set

        キャッシュを追加します。
        [args]
            :key        キー
            :value      キャッシュしたい値
            :timeout    有効期限

    ------------------------------"""
    def set(self, key, value, timeout=None):
        pass

    u"""-----------------------------
        Shimehari.core.cachestore.BaseCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        add

        キャッシュを追加します。
        [args]
            :key        キー
            :value      値
            :timeout    有効期限

    ------------------------------"""
    def add(self, key, value, timeout=None):
        pass

    u"""-----------------------------
        Shimehari.core.cachestore.BaseCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        setMany

        複数のオブジェクトをまとめてキャッシュします。
        [args]
            :mapping    キャッシュしたいオブジェクトたちのキーマッピング
            :timeout    有効期限

    ------------------------------"""
    def setMany(self, mapping, timeout=None):
        for k, v in _items(mapping):
            self.set(k, v, timeout)

    u"""-----------------------------
        Shimehari.core.cachestore.BaseCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        deleteMany

        複数のキーからまとめてキャッシュを消去します。
        [args]
            :keys キー達

    ------------------------------"""
    def deleteMany(self, *keys):
        [self.delete(k) for k in keys]

    u"""-----------------------------
        Shimehari.core.cachestore.BaseCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        clear

        ストアの中のキャッシュを全て消去します
        [args]
            :key

    ------------------------------"""
    def clear(self):
        pass


class NullCacheStore(BaseCacheStore):
    pass


u"""-----------------------------
    Shimehari.core.cachestore.SimpleCacheStore
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    メモリ上に突っ込むシンプルなキャッシュ

    ------------------------------"""


class SimpleCacheStore(BaseCacheStore):
    def __init__(self, threshold=500, default_timeout=300):
        BaseCacheStore.__init__(self, default_timeout)
        self._cache = {}
        self.clear = self._cache.clear
        self._threshold = threshold

    u"""-----------------------------
        Shimehari.core.cachestore._prune
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        _prune

        有効期限の切れたキャッシュを消します。

    ------------------------------"""
    def _prune(self):
        #キャッシュの数が増えた時消す
        if len(self._cache) > self._threshold:
            now = time()
            for ind, (key, (expires, _)) in enumerate(self._cache.items()):
                if expires <= now or ind % 3 == 0:
                    self._cache.pop(key, None)

    u"""-----------------------------
        Shimehari.core.cachestore.SimpleCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        get

        キャッシュを取得します。

    ------------------------------"""
    def get(self, key):
        now = time()
        expires, value = self._cache.get(key, (0, None))
        if expires > now:
            return msg.loads(value)

    u"""-----------------------------
        Shimehari.core.cachestore.SimpleCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        set

        キャッシュを追加します。

    ------------------------------"""
    def set(self, key, value, timeout=None):
        if timeout is None:
            timeout = self.default_timeout

        self._prune()
        self._cache[key] = (time() + timeout, msg.dumps(value, msg.HIGHEST_PROTOCOL))

    u"""-----------------------------
        Shimehari.core.cachestore.SimpleCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        add

        キーをチェックして存在していなかった時キャッシュを追加します。

    ------------------------------"""
    def add(self, key, value, timeout=None):
        if timeout is None:
            timeout = self.default_timeout

        if len(self._cache) > self._threshold:
            self._prune()
        item = (time() + timeout, msg.dumps(value, msg.HIGHEST_PROTOCOL))
        self._cache.setdefault(key, item)

    u"""-----------------------------
        Shimehari.core.cachestore.SimpleCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        delete

        指定されたキーのキャッシュを削除します。

    ------------------------------"""
    def delete(self, key):
        self._cache.pop(key, None)


import re
from shimehari.core.helpers import importPreferredMemcachedClient


u"""-----------------------------
    Shimehari.core.cachestore.MemcachedCacheStore
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Memcache を使ったキャッシュストア
    ------------------------------"""


_testMemcachedKey = re.compile(r'[^\x00-\x21\xff]{1,250}$').match


class MemcachedCacheStore(BaseCacheStore):
    def __init__(self, servers=None, default_timeout=300, key_prefix=None):
        BaseCacheStore.__init__(self, default_timeout)
        if servers is None or isinstance(servers, (list, tuple)):
            if servers is None:
                servers = ['127.0.0.1:11211']
            self._client = importPreferredMemcachedClient(servers)
            if servers is None:
                raise RuntimeError('emcache nai jan')
        else:
            self._client = servers

        self.key_prefix = key_prefix

    u"""-----------------------------
        Shimehari.core.cachestore.MemcachedCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        get

        指定したキーからキャッシュを取得します。
        [args]
            :key キー
        [return]
            キャッシュ

    ------------------------------"""
    def get(self, key):
        if isinstance(key, unicode):
            key = key.encode('utf-8')
        if self.key_prefix:
            key = self.key_prefix + key

        if _testMemcachedKey(key):
            return self._client.get(key)

    u"""-----------------------------
        Shimehari.core.cachestore.MemcachedCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        getDict

        複数のキーからまとめてキャッシュを取得し、dict で返します。
        [args]
            :keys   キー
        [return]
            :dict   キャッシュ

    ------------------------------"""
    def getDict(self, *keys):
        keyMap = {}
        haveEncodedKeys = False
        for key in keys:
            if isinstance(key, unicode):
                encodedKey = key.encode('utf-8')
                haveEncodedKeys = True
            else:
                encodedKey = key
            if self.key_prefix:
                encodedKey = self.key_prefix + encodedKey
            if _testMemcachedKey(encodedKey):
                keyMap[encodedKey] = key
        d = rv = self._client.get_multi(keyMap.keys())
        if haveEncodedKeys or self.key_prefix:
            rv = {}
            for k, v in d.iteritems():
                rv[keyMap[k]] = v
        if len(rv) < len(keys):
            for k in keys:
                if k not in rv:
                    rv[k] = None
        return rv

    u"""-----------------------------
        Shimehari.core.cachestore.MemcachedCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        getMany

        複数のキーからまとめてキャッシュを取得します。

    ------------------------------"""
    def getMany(self, *keys):
        d = self.getDict(*keys)
        return [d[k] for k in keys]

    u"""-----------------------------
        Shimehari.core.cachestore.MemcachedCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        set

        キャッシュを追加します。
        [args]
            :key        キャッシュのキー
            :value      キャッシュしたい値
            :timeout    有効期限

    ------------------------------"""
    def set(self, key, value, timeout=None):
        if timeout is None:
            timeout = self.default_timeout
        if isinstance(key, unicode):
            key = key.encode('utf-8')
        if self.key_prefix:
            key = self.key_prefix + key
        self._client.set(key, value, timeout)

    u"""-----------------------------
        Shimehari.core.cachestore.MemcachedCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        setMany

        まとめて追加

    ------------------------------"""
    def setMany(self, mapping, timeout=None):
        if timeout is None:
            timeout = self.default_timeout
        newMap = {}
        for k, v in _items(mapping):
            if isinstance(k, unicode):
                k = k.encode('utf-8')
            if self.key_prefix:
                k = self.key_prefix + k
            newMap[k] = v
        self._client.set_multi(newMap, timeout)

    u"""-----------------------------
        Shimehari.core.cachestore.MemcachedCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        add

        キャッシュを上書き

    ------------------------------"""
    def add(self, key, value, timeout=None):
        if timeout is None:
            timeout = self.default_timeout
        if isinstance(key, unicode):
            key = key.encode('utf-8')
        if self.key_prefix:
            key = self.key_prefix + key
        self._client.add(key, value, timeout)

    u"""-----------------------------
        Shimehari.core.cachestore.MemcachedCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        delete

        キーで指定したキャッシュを削除
        [args]
            :key    削除したいキャッシュのキー

    ------------------------------"""
    def delete(self, key):
        if isinstance(key, unicode):
            key = key.encode('utf-8')
        if self.key_prefix:
            key = self.key_prefix + key
        if _testMemcachedKey(key):
            self._client.delete(key)

    u"""-----------------------------
        Shimehari.core.cachestore.MemcachedCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        deleteMany

        まとめて削除

    ------------------------------"""
    def deleteMany(self, *keys):
        newKeys = []
        for k in keys:
            if isinstance(k, unicode):
                k = k.encode('utf-8')
            if self.key_prefix:
                k = self.key_prefix + k
            if _testMemcachedKey(k):
                newKeys.append(k)
        self._client.delete_multi(newKeys)

    def clear(self):
        self._client.flush_all()


u"""-----------------------------
    Shimehari.core.cachestore.FileSystemCacheStore
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    FileSystem を使ったキャッシュストア

------------------------------"""


_fsTransactionSuffix = '.__sakekasu'


class FileSystemCacheStore(BaseCacheStore):
    def __init__(self, cacheDir, threshold=500, default_timeout=300, mode=0600):
        BaseCacheStore.__init__(self, default_timeout)
        self._cacheDir = cacheDir
        self._threshold = threshold
        self._mode = mode

        if not os.path.exists(self._cacheDir):
            os.makedirs(self._cacheDir)

    u"""-----------------------------
        Shimehari.core.cachestore.FileSystemCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        _getFileList

        キャッシュディレクトリ内のキャッシュ一覧を返します。

    ------------------------------"""
    def _getFileList(self):
        return [os.path.join(self._cacheDir, fn) for fn in os.listdir(self._cacheDir) if not fn.endswith(_fsTransactionSuffix)]

    u"""-----------------------------
        Shimehari.core.cachestore.FileSystemCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        _getFileName

        ファイル名を返します。

    ------------------------------"""
    def _getFileName(self, key):
        hash = md5(key).hexdigest()
        return os.path.join(self._cacheDir, hash)

    u"""-----------------------------
        Shimehari.core.cachestore.FileSystemCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        _prune

        有効期限の切れたキャッシュを消します。

    ------------------------------"""
    def _prune(self):
        entries = self._getFileList()
        if len(entries) > self._threshold:
            now = time()
            for ind, fn in enumerate(entries):
                remove = None
                f = None
                try:
                    try:
                        f = open(fn, 'rb')
                        expires = msg.load(f)
                        remove = expires <= now or ind % 3 == 0
                    except (IOError, OSError):
                        pass
                    finally:
                        if f is not None:
                            f.close()
                except Exception:
                    pass

                if remove:
                    try:
                        os.remove(fn)
                    except (IOError, OSError):
                        pass

    u"""-----------------------------
        Shimehari.core.cachestore.FileSystemCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        get

        指定されたキーからキャッシュを取得します。
        [args]
            :key    取得したいキャッシュのキー
        [return]
            :value  キャッシュ

    ------------------------------"""
    def get(self, key):
        fn = self._getFileName(key)
        try:
            f = open(fn, 'rb')
            try:
                if msg.load(f) >= time():
                    return msg.load(f)
            finally:
                f.close()
            os.remove(fn)
        except Exception:
            return None

    u"""-----------------------------
        Shimehari.core.cachestore.FileSystemCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        set

        新たなキャッシュを追加します。
        [args]
            :key        キー
            :value      追加したい値
            :timeout    有効期限

    ------------------------------"""
    def set(self, key, value, timeout=None):
        if timeout is None:
            timeout = self.default_timeout
        fn = self._getFileName(key)
        self._prune()
        try:
            fd, tmp = tempfile.mkstemp(suffix=_fsTransactionSuffix, dir=self._cacheDir)
            f = os.fdopen(fd, 'wb')
            try:
                msg.dump(int(time() + timeout), f, 1)
                msg.dump(value, f, msg.HIGHEST_PROTOCOL)
            finally:
                f.close()
            os.rename(tmp, fn)
            os.chmod(fn, self._mode)
        except (IOError, OSError), e:
            pass

    u"""-----------------------------
        Shimehari.core.cachestore.FileSystemCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        add

        キャッシュを追加します。
        [args]
            :key        追加したいキー
            :value      追加したい値
            :timeout    有効期限

    ------------------------------"""
    def add(self, key, value, timeout=None):
        fn = self._getFileName(key)
        if not os.path.exists(fn):
            self.set(key, value, timeout)

    u"""-----------------------------
        Shimehari.core.cachestore.FileSystemCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        delete

        指定されたキャッシュを削除します
        [args]
            :key 削除したいキャッシュのキー

    ------------------------------"""
    def delete(self, key):
        try:
            os.remove(self._getFileName(key))
        except (IOError, OSError):
            pass

    u"""-----------------------------
        Shimehari.core.cachestore.FileSystemCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        clear

        キャッシュを全て削除します。

    ------------------------------"""
    def clear(self):
        for fn in self._getFileList():
            try:
                os.remove(fn)
            except (ImportError, OSError):
                pass


u"""-----------------------------
    Shimehari.core.cachestore.RedisCacheStore
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    Redis を使ったキャッシュストア

------------------------------"""


class RedisCacheStore(BaseCacheStore):
    def __init__(self, host='localhost', port=6379, passwd=None, default_timeout=300, key_prefix=None):
        BaseCacheStore.__init__(self, default_timeout)
        if isinstance(host, basestring):
            try:
                import redis
            except ImportError, e:
                raise RuntimeError('no redis module')
            self._client = redis.Redis(host=host, port=port, password=passwd)
        else:
            self._client = host
        self.key_prefix = key_prefix or ''

    u"""-----------------------------
        Shimehari.core.cachestore.RedisCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        dump

        与えられた値をシリアライズします。
        [args]
            :value  シリアライズしたい値

    ------------------------------"""
    def dump(self, value):
        t = type(value)
        if t is int or t is long:
            return str(value)
        else:
            return '!' + msg.dumps(value)

    u"""-----------------------------
        Shimehari.core.cachestore.RedisCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        load

        シリアライズされたものを復元します。
        [args]
            :value  復元したい項目

    ------------------------------"""
    def load(self, value):
        if value is None:
            return None
        if value.startswith('!'):
            return msg.loads(value[1:])
        try:
            return int(value)
        except ValueError:
            return value

    u"""-----------------------------
        Shimehari.core.cachestore.RedisCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        get

        指定されたキーのキャッシュを取得します
        [args]
            :key    キー

        [return]
            キャッシュ

    ------------------------------"""
    def get(self, key):
        return self.load(self._client.get(self.key_prefix + key))

    u"""-----------------------------
        Shimehari.core.cachestore.RedisCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        getMany

        複数のキーからまとめて取得します。
        [args]
            :keys   キー達
        [return]
            キャッシュ

    ------------------------------"""
    def getMany(self, *keys):
        if self.key_prefix:
            keys = [self.key_prefix + k for k in keys]
        return [self.load(x) for x in self._client.mget(keys)]

    u"""-----------------------------
        Shimehari.core.cachestore.RedisCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        set

        キャッシュを追加します。
        [args]
            :key        キー
            :value      キャッシュしたい項目
            :timeout    有効期限

    ------------------------------"""
    def set(self, key, value, timeout=None):
        if timeout is None:
            timeout = self.default_timeout
        dump = self.dump(value)
        self._client.setex(self.key_prefix + key, dump, timeout)

    u"""-----------------------------
        Shimehari.core.cachestore.RedisCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        setMany

        まとめて複数のキャッシュを追加します
        [args]
            :mapping    まとめて追加したい項目
            :timeout    有効期限

    ------------------------------"""
    def setMany(self, mapping, timeout=None):
        if timeout is None:
            timeout = self.default_timeout
        pipe = self._client.pipeline()
        for k, v in _items(mapping):
            self.dump(v)
            pipe.setex(self.key_prefix + k, v, timeout)
        pipe.execute()

    u"""-----------------------------
        Shimehari.core.cachestore.RedisCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        add

        指定されたキーのキャッシュを削除します。
        [args]
            :key

    ------------------------------"""
    def add(self, key, value, timeout=None):
        if timeout is None:
            timeout = self.default_timeout
        self.dump(value)
        added = self._client.setnx(self.key_prefix + key, value)
        if added:
            self._client.expire(self.key_prefix + key, timeout)

    u"""-----------------------------
        Shimehari.core.cachestore.RedisCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        delete

        指定されたキーのキャッシュを削除します。
        [args]
            :key    キー

    ------------------------------"""
    def delete(self, key):
        self._client.delete(self.key_prefix + key)

    u"""-----------------------------
        Shimehari.core.cachestore.RedisCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        deleteMany

        複数のキーからまとめてキャッシュを削除します。
        [args]
            :keys   キー

    ------------------------------"""
    def deleteMany(self, *keys):
        if not keys:
            return
        if self.key_prefix:
            keys = [self.key_prefix + k for k in keys]
        self._client.delete(*keys)

    u"""-----------------------------
        Shimehari.core.cachestore.RedisCacheStore
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        clear

        キャッシュストアの中身を全て消去します。

    ------------------------------"""
    def clear(self):
        if self.key_prefix:
            keys = self._client.keys(self.key_prefix + '*')
            if keys:
                self._client.delete(*keys)
        self._client.flushdb()
